# -*- coding: utf-8 -*-
#
# File: overrides.py
#
# GNU General Public License (GPL)
#

from AccessControl import Unauthorized
from ftw.labels.browser.labeling import Labeling
from ftw.labels.browser.labelsjar import LabelsJar
from ftw.labels.interfaces import ILabeling
from ftw.labels.interfaces import ILabelSupport
from ftw.labels.jar import LabelJar
from ftw.labels.portlets.labeljar import Renderer as FTWLabelsRenderer
from ftw.labels.viewlets.labeling import LabelingViewlet
from plone import api
from Products.PloneMeeting.config import PMMessageFactory as _
from Products.PloneMeeting.utils import notifyModifiedAndReindex
from zope.annotation import IAnnotations


class PMFTWLabelsRenderer(FTWLabelsRenderer):
    """ """
    @property
    def available(self):
        """ """
        available = super(PMFTWLabelsRenderer, self).available
        return available and \
            'labels' in self.context.getUsedItemAttributes() and \
            self.request.get('pageName', None) == 'data'


class PMLabelJar(LabelJar):

    def add(self, title, color, by_user):
        """Override to invalidate relevant cache."""
        notifyModifiedAndReindex(self.context)
        return super(PMLabelJar, self).add(title, color, by_user)

    def remove(self, label_id):
        """Protect against removal of used labels."""
        cfg = self.context
        brains = api.content.find(portal_type=cfg.getItemTypeName(), labels=label_id)
        if brains:
            api.portal.show_message(
                _('This label can not be removed as it is used by some items, for example ${item_url}',
                  mapping={'item_url': brains[0].getURL()}),
                type='error',
                request=self.context.REQUEST)
            return self.context.REQUEST.RESPONSE.redirect(
                self.context.REQUEST['HTTP_REFERER'])
        notifyModifiedAndReindex(self.context)
        return super(PMLabelJar, self).remove(label_id)

    def update(self, label_id, title, color, by_user):
        """ """
        notifyModifiedAndReindex(self.context)
        return super(PMLabelJar, self).update(label_id, title, color, by_user)


class PMLabelsJar(LabelsJar):
    """ """

    def remove(self):
        """Redirect to HTTP_REFERER."""
        super(PMLabelsJar, self).remove()
        return self._redirect()


class PMLabeling(Labeling):
    """ """
    def __init__(self, context, request):
        super(PMLabeling, self).__init__(context, request)
        self.tool = api.portal.get_tool('portal_plonemeeting')
        self.cfg = self.tool.getMeetingConfig(self.context)

    def available_labels(self, modes=('view', 'edit')):
        # cache in request
        cache_key = "ftw-labeling-cache-{0}-{1}".format(self.context.UID(), "-".join(modes))
        cache = IAnnotations(self.request)
        value = cache.get(cache_key, None)
        if value is None:
            labeling = ILabeling(self.context)
            value = labeling.filter_manageable_labels(
                labeling.available_labels(), modes=modes)
            cache[cache_key] = value
        return value

    def update(self):
        """ """
        if not self.can_edit:
            raise Unauthorized
        return super(PMLabeling, self).update()

    def pers_update(self):
        """ """
        if not self.can_personal_edit:
            raise Unauthorized
        return super(PMLabeling, self).pers_update()

    @property
    def can_edit(self):
        return bool(self.available_labels(modes=('edit', ))[1])

    @property
    def can_personal_edit(self):
        return 1


class PMFTWLabelsLabelingViewlet(LabelingViewlet):
    """ """

    def __init__(self, context, request, view, manager=None):
        super(PMFTWLabelsLabelingViewlet, self).__init__(context, request, view, manager=None)
        self.tool = api.portal.get_tool('portal_plonemeeting')
        self.cfg = self.tool.getMeetingConfig(self.context)

    @property
    def available(self):
        """ """
        # for PloneMeeting
        if 'labels' not in self.cfg.getUsedItemAttributes():
            return False

        # override to avoid several calls to self.available_labels
        available_labels = self.available_labels
        if not available_labels:
            return False
        if 'portal_factory' in self.context.absolute_url():
            return False
        if not ILabelSupport.providedBy(self.context):
            return False
        if not available_labels[0] and not available_labels[1]:
            return False
        return True

    @property
    def available_labels(self):
        return self.context.restrictedTraverse('@@labeling').available_labels()

    @property
    def can_edit(self):
        return self.context.restrictedTraverse('@@labeling').can_edit

    @property
    def can_personal_edit(self):
        return self.context.restrictedTraverse('@@pers-labeling').can_personal_edit



# -*- coding: utf-8 -*-

from natsort import humansorted
from operator import attrgetter
from Products.PloneMeeting.external.forms import projects_default
from Products.PloneMeeting.external.forms import tasks_default
from Products.PloneMeeting.external.utils import send_json_request
from zope.i18n import translate
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class BaseVisionVocabulary(object):

    implements(IVocabularyFactory)

    def __call__(self, context, endpoint, selected_values=[]):
        """ """
        terms = []
        for info in send_json_request(endpoint):
            # concatenate element's project, parent and name for term title
            res = []
            # store also non HTML raw value to be able to sort it correctly
            raw_res = []
            project_name = info['infos'].get('project', {}).get('name')
            if project_name:
                res.append("<span class='discreet'>%s</span>" % project_name)
                raw_res.append(project_name)
            parent_name = info['infos']['parent'].get('name')
            if parent_name:
                res.append("<span class='discreet'>%s</span>" % parent_name)
                raw_res.append(parent_name)
            res.append(info['name'])
            raw_res.append(info['name'])
            title = u" / ".join(res)
            if info['id'] in selected_values:
                title += u' [%s]' % translate(
                    'Linked', domain="PloneMeeting", context=context.REQUEST)
            raw_title = u" / ".join(raw_res)
            term = SimpleTerm(info['id'], info['id'], title)
            term.raw_title = raw_title
            terms.append(term)
        terms = humansorted(terms, key=attrgetter('raw_title'))
        return SimpleVocabulary(terms)


class VisionProjectsVocabulary(BaseVisionVocabulary):

    implements(IVocabularyFactory)

    def __call__(self, context):
        """ """
        selected_values = projects_default(context)
        return super(VisionProjectsVocabulary, self).__call__(
            context, endpoint="projects", selected_values=selected_values)


VisionProjectsVocabularyFactory = VisionProjectsVocabulary()


class VisionTasksVocabulary(BaseVisionVocabulary):

    implements(IVocabularyFactory)

    def __call__(self, context):
        """ """
        selected_values = tasks_default(context)
        return super(VisionTasksVocabulary, self).__call__(
            context, endpoint="tasks", selected_values=selected_values)


VisionTasksVocabularyFactory = VisionTasksVocabulary()



# -*- coding: utf-8 -*-

from Products.Archetypes.event import ObjectEditedEvent
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from zope.event import notify


class Migrate_To_4201(Migrator):

    def _fixWFsUsingReturnedToProposingGroupWFAdaptation(self):
        '''WFAdaptation returned_to_proposing_group was not correctly set up.'''
        logger.info('Updating every MeetingConfigs "returned_to_proposing_group" configuration...')
        for cfg in self.tool.objectValues('MeetingConfig'):
            returned_states = [state_id for state_id in cfg.getItemWorkflow(True).states
                               if state_id.startswith('returned_to_proposing_group')]
            if returned_states:
                # reload config, this will apply WFA correctly
                notify(ObjectEditedEvent(cfg))
                # update WF permissions for items in relevant states
                item_wf = cfg.getItemWorkflow(True)
                brains = self.catalog(
                    portal_type=cfg.getItemTypeName(), review_state=returned_states)
                for brain in brains:
                    item = brain.getObject()
                    item_wf.updateRoleMappingsFor(item)
                    logger.info('Updating WF role mappings for item at %s' % brain.getPath())
        logger.info('Done.')

    def _disableWscForCKeditor(self):
        '''The link 'Check spell' in Scayt button is broken, hide it.'''
        logger.info('CKeditor, adding "wsc" to the removePlugins property...')
        cke_props = self.portal.portal_properties.ckeditor_properties
        cke_props.removePlugins = (u'wsc',)
        logger.info('Done.')

    def run(self, extra_omitted=[], from_migration_to_4200=False):
        logger.info('Migrating to PloneMeeting 4201...')

        self._disableWscForCKeditor()

        # refresh if we are not coming from migration to 4200,
        # in this case it was already refreshed
        if not from_migration_to_4200:
            self._fixWFsUsingReturnedToProposingGroupWFAdaptation()


def migrate(context):
    '''This migration function will:

       1) Fix permissions for items in state 'returned_to_proposing_group';
       2) Remove the 'wsc' plugin from CKeditor.
    '''
    migrator = Migrate_To_4201(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-
#
# File: behaviors.py
#
# GNU General Public License (GPL)
#

from plone.app.textfield import RichText
from plone.autoform.interfaces import IFormFieldProvider
from plone.directives import form
from plone.supermodel import model
from Products.PloneMeeting.config import PMMessageFactory as _
from Products.PloneMeeting.widgets.pm_richtext import PMRichTextFieldWidget
from zope.interface import alsoProvides


class IAdviceAccountingCommitmentBehavior(model.Schema):

    form.order_before(advice_accounting_commitment='advice_reference')
    form.widget('advice_accounting_commitment', PMRichTextFieldWidget)
    advice_accounting_commitment = RichText(
        title=_(u"title_advice_accounting_commitment"),
        required=False,
        allowed_mime_types=(u"text/html", ))


alsoProvides(IAdviceAccountingCommitmentBehavior, IFormFieldProvider)



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from imio.helpers.security import fplog
from plone import api
from Products.Five import BrowserView
from Products.PloneMeeting.utils import _itemNumber_to_storedItemNumber
from Products.PloneMeeting.utils import _storedItemNumber_to_itemNumber
from zope.i18n import translate


def _to_integer(number):
    """
      Return the entire value of p_number :
      - 200 is 200;
      - 202 is 200;
      - 305 is 300.
    """
    return number / 100 * 100


def _is_integer(number):
    """Is p_number an integer item number or a subnumber?"""
    return not bool(number % 100)


def _compute_value_to_add(number):
    """ """
    number_is_integer = _is_integer(number)
    if number_is_integer:
        return 100
    else:
        return 1


def _use_same_integer(number1, number2):
    """Is p_number1 part using same integer as p_number2?
       200 is using same integer as 201, 202.
    """
    if _to_integer(number1) == _to_integer(number2):
        return True
    return False


class ChangeItemOrderView(BrowserView):
    """
      Manage the functionnality that change item order on a meeting.
      Change one level up/down or to a given p_moveNumber.
    """

    def __call__(self, moveType, wishedNumber=None):
        """
          Change the items order on a meeting.
          This is an unrestricted method so a MeetingManager can change items
          order even if some items are no more movable because decided
          (and so no more 'Modify portal content' on it).
          We double check that current user can actually may_change_items_order.
          Anyway, this method move an item, one level up/down or at a given position.
        """
        # we do this unrestrictively but anyway respect the Meeting.may_change_items_order
        meeting = self.context.getMeeting()

        if not meeting.wfConditions().may_change_items_order():
            raise Unauthorized

        oldIndex = self.context.getItemNumber()
        plone_utils = api.portal.get_tool('plone_utils')

        # Move the item up (-1), down (+1) or at a given position ?
        if moveType == 'number':
            try:
                float(wishedNumber)
                # In this case, wishedNumber specifies the new position where
                # the item must be moved.
            except (ValueError, TypeError):
                plone_utils.addPortalMessage(
                    translate(msgid='item_number_invalid',
                              domain='PloneMeeting',
                              context=self.request),
                    type='warning')
                return False

        nbOfItems = meeting.number_of_items()
        items = meeting.get_items(ordered=True, unrestricted=True)

        # Calibrate and validate moveValue
        if moveType == 'number':
            # we receive 2.1, 2.5 or 2.10 but we store 201, 205 and 210 so it is orderable integers
            moveNumber = _itemNumber_to_storedItemNumber(wishedNumber)
            moveNumberIsInteger = _is_integer(moveNumber)
            # Is this move allowed ?
            if moveNumber == oldIndex:
                plone_utils.addPortalMessage(
                    translate(msgid='item_did_not_move',
                              domain='PloneMeeting',
                              context=self.request),
                    type='warning')
                return False
            # check that moveNumber is not < 1 or not > next possible item
            # check that the moveNumber is valid, aka integer (4) or max 2 decimal number (4.1 or 4.13)
            # check also that if we use a subnumber, the previous exists (22.2 exists if we specified 22.3)
            # check finally that if we are moving an item to a subnumber,
            # the master exists (moving to 12.1, 12 has to exist)
            last_item = items[-1]
            last_item_number = last_item.getItemNumber()
            if (moveNumber < 100) or \
               (moveNumber > _to_integer(last_item_number + 100)) or \
               (not moveNumberIsInteger and len(wishedNumber.split('.')[1]) > 2) or \
               (not moveNumberIsInteger and
                (not meeting.get_item_by_number(moveNumber - 1) or
                 moveNumber - 1 == oldIndex)):
                plone_utils.addPortalMessage(
                    translate(msgid='item_illegal_move',
                              domain='PloneMeeting',
                              context=self.request),
                    type='warning')
                return False
        else:
            # down, must not be last
            # up, must not be first
            if (moveType == 'down' and self.context == items[-1]) or \
               (moveType == 'up' and self.context == items[0]):
                plone_utils.addPortalMessage(
                    translate(msgid='item_illegal_switch',
                              domain='PloneMeeting',
                              context=self.request),
                    type='warning')
                return False
            # 'down' or 'up' may not switch an integer and a subnumber
            currentIsInteger = _is_integer(self.context.getItemNumber())
            illegal_switch = False
            if moveType == 'down':
                nextIsInteger = _is_integer(self.context.getSiblingItem('next'))
                if (currentIsInteger and not nextIsInteger) or (not currentIsInteger and nextIsInteger):
                    illegal_switch = True
            elif moveType == 'up':
                previousIsInteger = _is_integer(self.context.getSiblingItem('previous'))
                if (currentIsInteger and not previousIsInteger) or (not currentIsInteger and previousIsInteger):
                    illegal_switch = True
            if illegal_switch:
                plone_utils.addPortalMessage(
                    translate(msgid='item_illegal_switch',
                              domain='PloneMeeting',
                              context=self.request),
                    type='warning')
                return False

        # Move the item
        if nbOfItems >= 2:
            if not moveType == 'number':
                # switch the items
                if moveType == 'up':
                    if oldIndex == 100:
                        # moving first item 'up', does not change anything
                        # actually it is not possible in the UI because the 'up'
                        # icon is not displayed on the first item
                        return False
                    otherNumber = self.context.getSiblingItem('previous')
                else:
                    # moveType == 'down'
                    otherNumber = self.context.getSiblingItem('next')
                otherItem = meeting.get_item_by_number(otherNumber)
                self.context.setItemNumber(otherItem.getItemNumber())
                otherItem.setItemNumber(oldIndex)
            else:
                # Move the item to an absolute position
                self.context.setItemNumber(moveNumber)
                # get items again now that context number was updated
                items = meeting.get_items(ordered=True, unrestricted=True)
                oldIndexIsInteger = _is_integer(oldIndex)
                oldIndexHasSubnumbers = meeting.get_item_by_number(oldIndex + 1)
                if moveNumber < oldIndex:
                    # We moved the item up
                    for item in items:
                        itemNumber = item.getItemNumber()
                        # moved item
                        if item == self.context:
                            # moving 4 to 2
                            if (oldIndexIsInteger and moveNumberIsInteger):
                                pass
                            # moving 4.1 to 2
                            elif (not oldIndexIsInteger and moveNumberIsInteger):
                                pass
                            # moving 4 to 2.1
                            elif (oldIndexIsInteger and not moveNumberIsInteger):
                                pass
                            elif (not oldIndexIsInteger and not moveNumberIsInteger):
                                # moving 3.1 to 2.2
                                pass
                        # item that was at the moveNumber position
                        elif itemNumber == moveNumber:
                            # moving 4 to 2
                            if (oldIndexIsInteger and moveNumberIsInteger):
                                item.setItemNumber(itemNumber + 100)
                            # moving 4.1 to 2
                            elif (not oldIndexIsInteger and moveNumberIsInteger):
                                item.setItemNumber(itemNumber + _compute_value_to_add(itemNumber))
                            # moving 4 to 2.1
                            elif (oldIndexIsInteger and not moveNumberIsInteger):
                                item.setItemNumber(itemNumber + _compute_value_to_add(itemNumber))
                            elif (not oldIndexIsInteger and not moveNumberIsInteger):
                                # moving 3.1 to 2.2
                                item.setItemNumber(itemNumber + 1)
                        # other items
                        else:
                            # moving 4 to 2
                            if (oldIndexIsInteger and moveNumberIsInteger) and \
                               (itemNumber < oldIndex and itemNumber > moveNumber):
                                item.setItemNumber(itemNumber + 100)
                            # moving 4.1 to 2
                            elif (not oldIndexIsInteger and moveNumberIsInteger) and itemNumber > moveNumber:
                                # subnumbers of oldIndex (4.2, 4.3, ...) must be decreased of 0.1
                                if _use_same_integer(itemNumber, oldIndex) and itemNumber > oldIndex:
                                    item.setItemNumber(itemNumber + 100 - _compute_value_to_add(itemNumber))
                                else:
                                    item.setItemNumber(itemNumber + 100)
                            # moving 4 to 2.1
                            elif (oldIndexIsInteger and not moveNumberIsInteger):
                                # moving master, decrease old subnumbers if > oldIndex
                                if _use_same_integer(itemNumber, oldIndex):
                                    item.setItemNumber(itemNumber - _compute_value_to_add(itemNumber))
                                # increase subnumbers > moveNumber
                                if _use_same_integer(itemNumber, moveNumber) and itemNumber > moveNumber:
                                    item.setItemNumber(itemNumber + _compute_value_to_add(itemNumber))
                                # decrease every number > oldIndex if oldIndex was not a master number
                                elif not oldIndexHasSubnumbers and itemNumber > oldIndex:
                                    item.setItemNumber(itemNumber - 100)
                            elif (not oldIndexIsInteger and not moveNumberIsInteger):
                                # manage when moving 3.1 to 2.2 or 3.4 to 3.2
                                # moving to another integer (3.1 to 2.2)
                                if not _use_same_integer(moveNumber, oldIndex):
                                    # decrease 3.2 to 3.1
                                    if _use_same_integer(itemNumber, oldIndex):
                                        if itemNumber > oldIndex:
                                            item.setItemNumber(itemNumber - 1)
                                    # increase 2.3 to 2.4
                                    elif _use_same_integer(itemNumber, moveNumber) and \
                                            itemNumber > moveNumber:
                                        item.setItemNumber(itemNumber + 1)
                                else:
                                    # moving to same integer (3.4 to 3.2)
                                    # increase itemNumber in between but itemNumber after
                                    # oldIndex does not change
                                    if _use_same_integer(itemNumber, moveNumber) and \
                                       itemNumber > moveNumber and \
                                       itemNumber < oldIndex:
                                        item.setItemNumber(itemNumber + 1)
                else:
                    # We moved the item down
                    for item in items:
                        itemNumber = item.getItemNumber()
                        if item == self.context:
                            # moving 2 to 4
                            if (oldIndexIsInteger and moveNumberIsInteger):
                                # if moving to last position, we need to remove 100 if > last_item_number
                                if moveNumber > last_item_number:
                                    item.setItemNumber(moveNumber - 100)
                            # moving 2.1 to 4
                            elif (not oldIndexIsInteger and moveNumberIsInteger):
                                pass
                            # moving 2 to 4.2
                            elif (oldIndexIsInteger and not moveNumberIsInteger):
                                if not oldIndexHasSubnumbers:
                                    item.setItemNumber(moveNumber - 100)
                            elif (not oldIndexIsInteger and not moveNumberIsInteger):
                                # moving 2.1 to 3.2
                                # (not oldIndexIsInteger and not moveNumberIsInteger)
                                pass
                        elif itemNumber == moveNumber:
                            # moving 2 to 4
                            if (oldIndexIsInteger and moveNumberIsInteger):
                                item.setItemNumber(itemNumber - _compute_value_to_add(itemNumber))
                            # moving 2.1 to 4
                            elif (not oldIndexIsInteger and moveNumberIsInteger):
                                item.setItemNumber(itemNumber + _compute_value_to_add(itemNumber))
                            # moving 2 to 4.2
                            elif (oldIndexIsInteger and not moveNumberIsInteger):
                                item.setItemNumber(itemNumber - 100 + _compute_value_to_add(itemNumber))
                            elif (not oldIndexIsInteger and not moveNumberIsInteger):
                                # moving 2.1 to 3.2
                                # increase 3.2 to 3.3
                                if not _use_same_integer(moveNumber, oldIndex):
                                    item.setItemNumber(itemNumber + 1)
                                else:
                                    # moving 2.1 to 2.4, decrease 2.4 to 2.3
                                    item.setItemNumber(itemNumber - 1)
                        else:
                            # moving 2 to 4
                            if (oldIndexIsInteger and moveNumberIsInteger) and \
                               (itemNumber > oldIndex and itemNumber < moveNumber):
                                item.setItemNumber(itemNumber - 100)
                            # moving 2.1 to 4
                            elif (not oldIndexIsInteger and moveNumberIsInteger) and itemNumber > oldIndex:
                                # decrease elements having same subnumber as oldIndex
                                if (_use_same_integer(itemNumber, oldIndex)):
                                    item.setItemNumber(itemNumber - _compute_value_to_add(itemNumber))
                                elif itemNumber > moveNumber:
                                    item.setItemNumber(itemNumber + 100)
                            # moving 2 to 4.2
                            elif (oldIndexIsInteger and not moveNumberIsInteger) and itemNumber > oldIndex:
                                # decrease from 1 but add + 0.1 to subnumbers > moveNumber
                                if (_use_same_integer(itemNumber, moveNumber)) and itemNumber > moveNumber:
                                    item.setItemNumber(itemNumber - 100 + _compute_value_to_add(itemNumber))
                                # decrease subnumbers of master we moved
                                elif (_use_same_integer(itemNumber, oldIndex)):
                                    item.setItemNumber(itemNumber - _compute_value_to_add(itemNumber))
                                elif not oldIndexHasSubnumbers:
                                    item.setItemNumber(itemNumber - 100)
                            elif (not oldIndexIsInteger and not moveNumberIsInteger):
                                # manage when moving 2.1 to 3.2 or 3.2 to 3.4
                                # moving to another integer (2.1 to 3.2)
                                if not _use_same_integer(moveNumber, oldIndex):
                                    # decrease 2.2 to 2.1
                                    if _use_same_integer(itemNumber, oldIndex) and itemNumber > oldIndex:
                                        item.setItemNumber(itemNumber - 1)
                                    # increase 3.2 to 3.3
                                    elif _use_same_integer(itemNumber, moveNumber) and \
                                            itemNumber > moveNumber:
                                        item.setItemNumber(itemNumber + 1)
                                else:
                                    # moving to same integer (3.2 to 3.4)
                                    # increase itemNumber in between but itemNumber after
                                    # oldIndex does not change
                                    if _use_same_integer(itemNumber, moveNumber) and \
                                       itemNumber < moveNumber and \
                                       itemNumber > oldIndex:
                                        item.setItemNumber(itemNumber - 1)

        # when items order on meeting changed, it is considered modified,
        # do this before update_item_references
        meeting.notifyModified()

        # add logging message to fingerpointing log
        extras = 'object={0} meeting={1} original_position={2} new_position={3}'.format(
            repr(self.context),
            repr(meeting),
            _storedItemNumber_to_itemNumber(oldIndex, forceShowDecimal=False),
            self.context.getItemNumber(for_display=True))
        fplog('change_item_order', extras=extras)

        # update item references starting from minus between oldIndex and new itemNumber
        meeting.update_item_references(start_number=min(oldIndex, self.context.getItemNumber()))
        return True



from Products.Five import BrowserView


class PloneMeetingMacros(BrowserView):
    """
      Manage macros used for PloneMeeting.
    """
    def callMacro(self, page, macro):
        """
          Call the given p_macro on given p_page (that is a BrowserView containing macros)
        """
        return self.context.unrestrictedTraverse(page)[macro]



# -*- coding: utf-8 -*-

from collective.contact.plonegroup.utils import get_plone_group_id
from imio.helpers.cache import get_current_user_id
from imio.helpers.cache import get_plone_groups_for_user
from imio.helpers.content import uuidToObject
from plone import api
from plone.app.layout.navigation.navtree import buildFolderTree
from plone.app.layout.navigation.navtree import NavtreeStrategyBase
from plone.memoize import ram
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.PloneMeeting.config import TOOL_FOLDER_ITEM_TEMPLATES
from zope.i18n import translate


class ItemTemplateView(BrowserView):
    '''
      This manage the overlay popup displayed when a user want to select an item template to create a new item.
    '''
    def __init__(self, context, request):
        super(BrowserView, self).__init__(context, request)
        self.context = context
        self.request = request
        self.tool = api.portal.get_tool('portal_plonemeeting')
        self.cfg = self.tool.getMeetingConfig(self.context)
        self.request.set('disable_border', 1)

    def __call__(self):
        """ """
        form = self.request.form
        templateUID = form.get('templateUID', None)
        cancelled = form.get('form.buttons.cancel', False)
        if templateUID:
            newItem = self.createItemFromTemplate(templateUID)
            newItemUrl = newItem.absolute_url() + '/edit'
            # remove title if we are adding an item using default item template
            default_template = self.cfg.get_default_item_template()
            if default_template and templateUID == default_template.UID():
                newItemUrl += '?title='
            return self.request.RESPONSE.redirect(newItemUrl)
        elif cancelled:
            # the only way to enter here is the popup overlay not to be shown
            # because while using the popup overlay, the jQ function take care
            # of hidding it while the Cancel button is hit
            return self.request.RESPONSE.redirect(form.get('form.HTTP_REFERER'))
        else:
            # compute and store templates tree so it can be used in several places
            # this is not done in the __init__ because the user is "Anonymous" in the __init__
            # and as we filter on "templateUsingGroup", we need a valid user...
            self.renderedTemplatesTree = self._patch_html_content(self._createTemplatesTree())
            return self.index()

    def _template_path_and_title(self, templateItem):
        """Return title of p_templateItem including name of
           subfolders until 'itemtemplates' folder."""
        titles = [templateItem.Title()]
        parent = templateItem.aq_parent
        while parent.getId() != TOOL_FOLDER_ITEM_TEMPLATES:
            titles.insert(0, parent.Title())
            parent = parent.aq_parent
        return ' / '.join(titles)

    def createItemFromTemplate(self, templateUID):
        '''The user wants to create an item from a item template that lies in
           this meeting configuration. Item id is in the request.'''
        templateItem = uuidToObject(templateUID, unrestricted=True)
        # Create the new item by duplicating the template item
        member_id = get_current_user_id()
        template_path_and_title = safe_unicode(self._template_path_and_title(templateItem))
        cloneEventActionLabel = translate(
            'create_meeting_item_from_template_label_comments',
            domain='imio.history',
            mapping={'template_path_and_title': template_path_and_title, },
            context=self.request)
        # if a proposingGroup is defined on itemTemplate and current user is creator
        # for this proposingGroup, we keep it
        keepProposingGroup = False
        proposingGroup = templateItem.getProposingGroup()
        if get_plone_group_id(proposingGroup, 'creators') in get_plone_groups_for_user():
            keepProposingGroup = True
        newItem = templateItem.clone(newOwnerId=member_id,
                                     cloneEventAction='create_meeting_item_from_template',
                                     cloneEventActionLabel=cloneEventActionLabel,
                                     destFolder=self.context,
                                     newPortalType=self.cfg.getItemTypeName(),
                                     keepProposingGroup=keepProposingGroup,
                                     keep_ftw_labels=True)
        # set _at_creation_flag to True so if user cancel first edit, it will be removed
        newItem._at_creation_flag = True
        return newItem

    def _getTemplatesTree(self):
        '''Create the structure of elements used to display the item templates tree to the item creators.
           We only want to show folders and items the current creator may use, so we do that in 2 steps :
           - a first catalog query that will find every items the creator may use,
             when found, we save the differents paths to these items so we will be able to
             not take into account folder that could exist in the configuration but that would
             be empty because no items into it;
           - when the query is build with list of paths to consider, we use buildFolderTree
             that will build a 'tree' dict with elements and children.'''
        # first query the catalog to see wich items the current user may select
        itemTemplatesPath = '/'.join(self.cfg.itemtemplates.getPhysicalPath())
        itemTemplates = self.cfg.getItemTemplates(as_brains=True, onlyActive=True, filtered=True)
        # we need to keep the itemTemplatesPath
        folderPathsToKeep = [itemTemplatesPath, ]
        # now compute folder paths so only these paths will be passed to buildFolderTree
        # and this method will not consider any other folders
        for itemTemplate in itemTemplates:
            folderPath = '/'.join(itemTemplate.getPath().split('/')[0:-1])
            # keep every folders of folderPath in case the itemtemplate is in a sub/sub/sub/...folder
            while folderPath not in folderPathsToKeep:
                folderPathsToKeep.append(folderPath)
                folderPath = '/'.join(folderPath.split('/')[0:-1])
        query = self.cfg._itemTemplatesQuery(onlyActive=True, filtered=True)
        # we want to query every Folders too
        query['portal_type'] = (query['portal_type'], 'Folder')
        query['path'] = {'query': folderPathsToKeep, 'depth': 1}
        # define a strategy so rootPath is managed ourself or folderTree
        # fails to compute it because query['path']['query'] is a list here...
        strategy = NavtreeStrategyBase()
        strategy.rootPath = itemTemplatesPath
        # remove useless query parameters
        strategy.supplimentQuery.pop('is_default_page', None)
        folderTree = buildFolderTree(self.context, None, query, strategy)
        # the single left problem is the fact that we could have empty folders
        # because we look in the itemTemplatesPath and it returns 'directly contained items' and
        # folders, we can not do anything else using a catalog query...
        # so check children of the root level, if it is an empty folder, we remove it...
        childrenToKeep = []
        for child in folderTree['children']:
            if child['item'].portal_type == 'Folder' and not child['children']:
                continue
            childrenToKeep.append(child)
        folderTree['children'] = childrenToKeep
        return folderTree

    def displayShowHideAllLinks(self):
        '''Used on the template, display links "Show all / Hide all" making
           it possible to 'expand' or 'collapse' the entire tree, this is only relevant
           if at least one folder to expand/collapse...
        '''
        return self.renderedTemplatesTree.count('class="folder"')

    def _createTemplatesTree_cachekey(method, self):
        '''cachekey method for self._createTemplatesTree.'''
        return repr(self.cfg), self.cfg.modified(), get_plone_groups_for_user()

    @ram.cache(_createTemplatesTree_cachekey)
    def _createTemplatesTree(self):
        # if only one folder at root, we expand it by default
        templatesTree = self._getTemplatesTree()
        atMostOneElementAtRoot = len(templatesTree['children']) < 2
        return self.recurse(children=templatesTree.get('children', []),
                            expandRootLevel=atMostOneElementAtRoot).strip()

    def _patch_html_content(self, html_content):
        """To be able to use caching, we need to
           change [baseUrl] after __call__ is rendered."""
        html_content = html_content.replace("[baseUrl]", self.context.absolute_url())
        return html_content

    recurse = ViewPageTemplateFile('templates/itemtemplates_tree_recurse.pt')



# -*- coding: utf-8 -*-

from collective.contact.core.content.held_position import HeldPosition
from collective.contact.core.content.held_position import IHeldPosition
from collective.contact.core.utils import get_gender_and_number
from collective.contact.core.utils import get_position_type_name
from collective.contact.core.vocabulary import get_directory
from collective.contact.core.vocabulary import NoDirectoryFound
from collective.contact.plonegroup.config import PLONEGROUP_ORG
from collective.contact.plonegroup.utils import get_own_organization
from collective.contact.widget.schema import ContactChoice
from collective.contact.widget.source import ContactSourceBinder
from plone.autoform import directives as form
from plone.dexterity.schema import DexteritySchemaPolicy
from plone.directives import form as directives_form
from plone.supermodel import model
from Products.PloneMeeting.config import PMMessageFactory as _
from Products.PloneMeeting.events import _invalidateAttendeesRelatedCache
from Products.PloneMeeting.utils import _prefixed_gn_position_name
from Products.PloneMeeting.utils import plain_render
from Products.PloneMeeting.utils import split_gender_and_number
from Products.PloneMeeting.utils import uncapitalize
from Products.PloneMeeting.widgets.pm_checkbox import PMCheckBoxFieldWidget
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.globalrequest import getRequest
from zope.i18n import translate

import html
import zope.schema


class IPMHeldPosition(IHeldPosition):
    """ """
    form.order_before(position_type='start_date')
    position_type = zope.schema.Choice(
        title=_("Position type"),
        description=_("Select a position type, correct label will be taken from this list "
                      "depending on person gender and context. If you need to add new position types, "
                      "it is defined on the directory at the root of contacts configuration "
                      "(element <a href='../../contacts/edit' target='_blank'>\"edit contacts\"</a>)."),
        vocabulary="PMPositionTypes",
        required=True,
    )

    form.order_after(secondary_position_type='position_type')
    secondary_position_type = zope.schema.Choice(
        title=_("Secondary position type"),
        description=_("Select a secondary position type if necessary. "
                      "Will work the same way as field \"Position type\" here above."),
        vocabulary="PMPositionTypes",
        required=False,
    )

    form.widget('usages', PMCheckBoxFieldWidget, multiple='multiple')
    usages = zope.schema.List(
        title=_("Usages"),
        value_type=zope.schema.Choice(
            vocabulary="Products.PloneMeeting.vocabularies.pmheldposition_usagesvocabulary"),
        required=False,
        default=[],
    )

    form.widget('defaults', PMCheckBoxFieldWidget, multiple='multiple')
    defaults = zope.schema.List(
        title=_("Defaults"),
        value_type=zope.schema.Choice(
            vocabulary="Products.PloneMeeting.vocabularies.pmheldposition_defaultsvocabulary"),
        required=False,
        default=[],
    )

    voting_group = ContactChoice(
        title=_("Voting group"),
        description=_("Voting group descr"),
        source=ContactSourceBinder(portal_type=("organization", "position")),
        required=False,
    )

    signature_number = zope.schema.Choice(
        title=_("Signature number"),
        description=_("If this contact is a default signer, select signature number."),
        vocabulary="Products.PloneMeeting.vocabularies.numbersvocabulary",
        required=False,
    )

    form.widget('represented_organizations', PMCheckBoxFieldWidget)
    represented_organizations = RelationList(
        title=_("Represented organizations"),
        default=[],
        description=_("Select organizations the current held position is representative for."),
        value_type=RelationChoice(
            vocabulary="PMSortedSelectedOrganizationsVocabulary"),
        required=False,
    )

    model.fieldset('held_position_app_parameters',
                   label=_(u"Application parameters"),
                   fields=['position', 'label', 'position_type',
                           'secondary_position_type',
                           'start_date', 'end_date',
                           'usages', 'defaults', 'voting_group',
                           'signature_number', 'represented_organizations'])


@directives_form.default_value(field=IHeldPosition['position'])
def position_default(data):
    return get_own_organization()


class PMHeldPosition(HeldPosition):
    """Override HeldPosition to add some fields and methods."""

    def get_label(self,
                  position_type_attr='position_type',
                  fallback_position_type_attr='position_type',
                  forced_position_type_value=None):
        """Override get_label to use position_type if label is empty."""
        value = self.label
        if not value or \
           (forced_position_type_value and forced_position_type_value != u'default'):
            values = self.gender_and_number_from_position_type(
                position_type_attr=position_type_attr,
                fallback_position_type_attr=fallback_position_type_attr,
                forced_position_type_value=forced_position_type_value)
            gender = self.get_person().gender
            if gender == 'M':
                value = values['MS']
            else:
                value = values['FS']
        return value

    def get_short_title(self,
                        include_usages=False,
                        include_defaults=False,
                        include_signature_number=False,
                        include_sub_organizations=True,
                        include_voting_group=False,
                        include_person_title=True,
                        abbreviate_firstname=False,
                        highlight=False,
                        position_type_attr='position_type',
                        fallback_position_type_attr='position_type',
                        forced_position_type_value=None):
        """Returns short name for held position :
           - the label if defined on held_position object or position title;
           - if position is in a sub organization, we display also sub-organization titles;
           - the person title.
           If p_include_usages and/or p_include_defaults is True, it is appendended
           at the end of the returned value.
           If highlight is True, we will display person_label and held_position_label in bold."""
        sub_organizations_label = u''
        # display sub-organizations title if any
        organization = self.get_organization()
        root_organization = organization.get_root_organization()
        sub_organizations = []
        if include_sub_organizations:
            if root_organization.getId() != PLONEGROUP_ORG:
                sub_organizations.append(root_organization)
            while organization != root_organization:
                sub_organizations.append(organization)
                organization = organization.aq_parent
        voting_group = None
        if include_voting_group:
            voting_group = self.voting_group and self.voting_group.to_object
        person_label = self.get_person_short_title(
            include_person_title=include_person_title,
            abbreviate_firstname=abbreviate_firstname)
        held_position_label = self.get_label(
            position_type_attr=position_type_attr,
            fallback_position_type_attr=fallback_position_type_attr,
            forced_position_type_value=forced_position_type_value) or \
            translate(
                'No label defined on held position',
                domain='PloneMeeting',
                context=getRequest(),
                default='No label defined on held position')
        if highlight:
            person_label = u'<b>{0}</b>'.format(html.escape(person_label))
            held_position_label = u'<b>{0}</b>'.format(html.escape(held_position_label))
        res = ''
        if sub_organizations:
            sub_organizations_label = u"{0}".format(u"➔".join(
                [sub_organization.title for sub_organization in sub_organizations]))
            res = u"{0}, {1} ({2})".format(
                person_label, held_position_label, sub_organizations_label)
        else:
            res = u"{0}, {1}".format(person_label, held_position_label)
        if voting_group:
            # do not include voting_group label if it is the same
            # as the sub_organizations label
            if not sub_organizations or sub_organizations_label != voting_group.title:
                res = u"{0} ({1})".format(
                    res, voting_group.title)
        if include_usages:
            res = res + u" ({0}: {1})".format(
                translate("Usages", domain="PloneMeeting", context=self.REQUEST),
                plain_render(self, 'usages') or '-')
        if include_defaults:
            res = res + u" ({0}: {1})".format(
                translate("Defaults", domain="PloneMeeting", context=self.REQUEST),
                plain_render(self, 'defaults') or '-')
        if include_signature_number:
            pattern = u" ({0}: {1})"
            signature_number = plain_render(self, 'signature_number')
            if highlight and signature_number:
                pattern = u" (<span class='highlightValue'>{0}: {1}</span>)"
            res = res + pattern.format(
                translate("Signature number", domain="PloneMeeting", context=self.REQUEST),
                signature_number or '-')
        return res

    def get_person_short_title(self,
                               include_person_title=False,
                               abbreviate_firstname=False,
                               include_held_position_label=False,
                               position_type_attr='position_type',
                               fallback_position_type_attr='position_type',
                               forced_position_type_value=None):
        """ """
        person = self.get_person()
        firstname = person.firstname
        if abbreviate_firstname:
            firstname = person.firstname_abbreviated or person.firstname
        person_title = u''
        if include_person_title:
            person_title = u'{0} '.format(person.person_title)
        person_held_position_label = u''
        if include_held_position_label:
            held_position_label = self.get_label(
                position_type_attr=position_type_attr,
                fallback_position_type_attr=fallback_position_type_attr,
                forced_position_type_value=forced_position_type_value) or u''
            person_held_position_label = u", {0}".format(held_position_label)
        return u'{0}{1} {2}{3}'.format(person_title, firstname, person.lastname, person_held_position_label)

    def gender_and_number_from_position_type(self,
                                             position_type_attr='position_type',
                                             fallback_position_type_attr='position_type',
                                             forced_position_type_value=None):
        """Split the position_type and generates a dict with gender and number possibilities."""
        value = forced_position_type_value or \
            (position_type_attr and getattr(self, position_type_attr, None)) or \
            (fallback_position_type_attr and getattr(self, fallback_position_type_attr, None))
        if value:
            try:
                directory = get_directory(self)
                value = get_position_type_name(directory, value)
            except NoDirectoryFound:
                pass
                # in some case like element creation, self does not
                # have acquisition, in this case, we pass
        return split_gender_and_number(value)

    def get_prefix_for_gender_and_number(self,
                                         include_value=False,
                                         include_person_title=False,
                                         use_by=False,
                                         use_to=False,
                                         position_type_attr='position_type',
                                         fallback_position_type_attr='position_type',
                                         forced_position_type_value=None):
        """Get prefix to use depending on given value."""
        res = u''
        value = self.get_label(position_type_attr=position_type_attr,
                               fallback_position_type_attr=fallback_position_type_attr,
                               forced_position_type_value=forced_position_type_value)
        if not value:
            return res

        values = {k: v for k, v in self.gender_and_number_from_position_type(
                  position_type_attr,
                  fallback_position_type_attr,
                  forced_position_type_value).items()
                  if v == value}

        res = (values or include_value) and _prefixed_gn_position_name(
            get_gender_and_number([self.get_person()], use_by=use_by, use_to=use_to),
            value, include_value=include_value) or u''

        if include_person_title:
            # we lowerize first letter of res so we have "Madame la directrice"
            res = uncapitalize(res)
            res = u'{0} {1}'.format(self.person_title, res)
        return res

    def _invalidateCachedMethods(self):
        '''Clean cache for vocabularies using held_positions.'''
        _invalidateAttendeesRelatedCache()


class PMHeldPositionSchemaPolicy(DexteritySchemaPolicy):
    """ """

    def bases(self, schemaName, tree):
        return (IPMHeldPosition, )



# -*- coding: utf-8 -*-

from imio.helpers.setup import load_type_from_package
from persistent.mapping import PersistentMapping
from Products.Archetypes.event import ObjectEditedEvent
from Products.CMFPlone.utils import base_hasattr
from Products.PloneMeeting.content.meeting import IMeeting
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from Products.PloneMeeting.setuphandlers import _configurePortalRepository
from zope.event import notify


class Migrate_To_4204(Migrator):

    def _reloadItemTemplateAndRecurringTypes(self):
        """Reload MeetingItemTemplate/MeetingItemRecurring portal_types to fix
           the allowed_content_types to only accept "Image"."""
        logger.info('Reloading MeetingItemTemplate/MeetingItemRecurring portal_types...')
        # first update MeetingItemTemplate/MeetingItemRecurring base portal_types
        load_type_from_package('MeetingItemTemplate', 'Products.PloneMeeting:default')
        load_type_from_package('MeetingItemRecurring', 'Products.PloneMeeting:default')
        for cfg in self.tool.objectValues('MeetingConfig'):
            notify(ObjectEditedEvent(cfg))
        logger.info('Done.')

    def _initMeetingsItemAttendeesOrder(self):
        """Initialize the item_attendees_order attribute for existing meetings."""
        logger.info('Initializing "item_attendees_order" for every meetings...')
        brains = self.catalog(object_provides=IMeeting.__identifier__)
        for brain in brains:
            meeting = brain.getObject()
            if base_hasattr(meeting, "item_attendees_order"):
                continue
            meeting.item_attendees_order = PersistentMapping()
            meeting._p_changed = True
        logger.info('Done.')

    def _migrateMCYearlyInitMeetingNumber(self):
        """Boolean field yearlyInitMeetingNumber is not a list of choices called
           yearlyInitMeetingNumbers."""
        logger.info('Migrating field "yearlyInitMeetingNumber" to '
                    '"yearlyInitMeetingNumbers" for every MeetingConfigs...')
        for cfg in self.tool.objectValues('MeetingConfig'):
            if hasattr(cfg, "yearlyInitMeetingNumber"):
                if cfg.yearlyInitMeetingNumber is True:
                    cfg.setYearlyInitMeetingNumbers(("meeting_number", ))
                delattr(cfg, "yearlyInitMeetingNumber")
                # fields Meeting.meeting_number and Meeting.first_item_number
                # are now optionnal so select it by default
                used_attrs = list(cfg.getUsedMeetingAttributes())
                used_attrs.append("meeting_number")
                used_attrs.append("first_item_number")
                cfg.setUsedMeetingAttributes(used_attrs)
        logger.info('Done.')

    def run(self, extra_omitted=[], from_migration_to_4200=False):

        logger.info('Migrating to PloneMeeting 4204...')

        # not necessary if executing the full upgrade to 4200
        if not from_migration_to_4200:
            _configurePortalRepository()
            self._reloadItemTemplateAndRecurringTypes()
            self._initMeetingsItemAttendeesOrder()
            self.updateHolidays()
        self._migrateMCYearlyInitMeetingNumber()

        # ToolPloneMeeting.get_plone_groups_for_user parameter userId is now user_id
        self.updateTALConditions("get_plone_groups_for_user(userId",
                                 "get_plone_groups_for_user(user_id")
        # ToolPloneMeeting.get_plone_groups_for_user is deprecated
        # use imio.helpers.cache.get_plone_groups_for_user that is available in pm_utils
        self.updateTALConditions("tool.get_plone_groups_for_user(",
                                 "pm_utils.get_plone_groups_for_user(")
        self.updateTALConditions("portal_plonemeeting.get_plone_groups_for_user(",
                                 "pm_utils.get_plone_groups_for_user(")

        # remove field MeetingConfig.transitionsForPresentingAnItem
        self.cleanMeetingConfigs(field_names=['transitionsForPresentingAnItem'])
        logger.info('Done.')


def migrate(context):
    '''This migration function will:

       1) Configure portal_repository;
       2) Reload MeetingItemTemplate/Recurring portal_types;
       3) Init attribute item_attendees_order for every meetings.
    '''
    migrator = Migrate_To_4204(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-

from collective.documentgenerator.content.condition import ConfigurablePODTemplateCondition
from collective.eeafaceted.dashboard.content.pod_template import DashboardPODTemplateCondition
from plone import api


class PMConfigurablePODTemplateCondition(ConfigurablePODTemplateCondition):
    """
    """

    def _extra_expr_ctx(self):
        """ """
        extra_context = super(PMConfigurablePODTemplateCondition, self)._extra_expr_ctx()
        tool = api.portal.get_tool('portal_plonemeeting')
        cfg = tool.getMeetingConfig(self.context)
        extra_context.update({'tool': tool,
                              'cfg': cfg})
        return extra_context

    def evaluate_allowed_context(self, context):
        """
        Override so we may use a single 'MeetingItemXXX' portal_type
        to work with real MeetingItem, ItemTemplate and RecurringItem item type.
        """
        allowed_types = list(self.pod_template.pod_portal_types)
        tool = api.portal.get_tool('portal_plonemeeting')
        cfg = tool.getMeetingConfig(context)
        # add the ItemTemplate and RecurringItem to allowed_types if itemTypeName is allowed
        if cfg and cfg.getItemTypeName() in allowed_types:
            allowed_types.append(cfg.getItemTypeName(configType='MeetingItemTemplate'))
            allowed_types.append(cfg.getItemTypeName(configType='MeetingItemRecurring'))
        return not allowed_types or context.portal_type in allowed_types


class PMDashboardPODTemplateCondition(DashboardPODTemplateCondition):
    """ """

    def _extra_expr_ctx(self):
        """ """
        extra_context = super(PMDashboardPODTemplateCondition, self)._extra_expr_ctx()
        tool = api.portal.get_tool('portal_plonemeeting')
        cfg = tool.getMeetingConfig(self.context)
        extra_context.update({'tool': tool,
                              'cfg': cfg})
        return extra_context



# -*- coding: utf-8 -*-

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from plone import api
from Products.CMFCore.permissions import ReviewPortalContent
from Products.CMFCore.utils import _checkPermission
from Products.PloneMeeting.config import ADVICE_GIVEN_HISTORIZED_COMMENT
from Products.PloneMeeting.interfaces import IMeetingAdviceWorkflowActions
from Products.PloneMeeting.interfaces import IMeetingAdviceWorkflowConditions
from zope.interface import implements


class MeetingAdviceWorkflowConditions(object):
    '''Adapts a MeetingAdvice to interface IMeetingAdviceWorkflowConditions.'''
    implements(IMeetingAdviceWorkflowConditions)
    security = ClassSecurityInfo()

    def __init__(self, advice):
        self.context = advice
        self.request = advice.REQUEST

    def _get_workflow(self):
        '''Return the workflow object used by self.context.'''
        wfTool = api.portal.get_tool('portal_workflow')
        return wfTool.getWorkflowsFor(self.context)[0]

    security.declarePublic('mayGiveAdvice')

    def mayGiveAdvice(self):
        '''See doc in interfaces.py.'''
        return self.request.get('mayGiveAdvice', False)

    security.declarePublic('mayBackToAdviceInitialState')

    def mayBackToAdviceInitialState(self):
        '''See doc in interfaces.py.'''
        return self.request.get('mayBackToAdviceInitialState', False)

    security.declarePublic('mayCorrect')

    def mayCorrect(self, destinationState=None):
        '''See doc in interfaces.py.'''
        res = False
        if _checkPermission(ReviewPortalContent, self.context):
            res = True
        return res


InitializeClass(MeetingAdviceWorkflowConditions)


class MeetingAdviceWorkflowActions(object):
    '''Adapts a MeetingAdvice to interface IMeetingAdviceWorkflowActions.'''
    implements(IMeetingAdviceWorkflowActions)
    security = ClassSecurityInfo()

    def __init__(self, advice):
        self.context = advice
        self.request = advice.REQUEST

    security.declarePrivate('doCorrect')

    def doCorrect(self, stateChange):
        """
          This is an unique wf action called for every transitions beginning with 'backTo'.
          Most of times we do nothing, but in some case, we check the old/new state and
          do some specific treatment.
        """
        pass

    security.declarePrivate('doGiveAdvice')

    def doGiveAdvice(self, stateChange):
        """Historize the advice and save item's relevant data
           if MeetingConfig.historizeItemDataWhenAdviceIsGiven.
           Make sure also the 'advice_given_on' data is correct in item's adviceIndex."""
        # historize
        self.context.historize_if_relevant(ADVICE_GIVEN_HISTORIZED_COMMENT)
        # manage 'advice_given_on' dates
        parent = self.context.aq_parent
        advice_given_on = self.context.get_advice_given_on()
        toLocalizedTime = parent.restrictedTraverse('@@plone').toLocalizedTime
        parent.adviceIndex[self.context.advice_group]['advice_given_on'] = advice_given_on
        parent.adviceIndex[self.context.advice_group]['advice_given_on_localized'] = toLocalizedTime(advice_given_on)


InitializeClass(MeetingAdviceWorkflowActions)



from zope.i18n import translate
from zope.publisher.browser import BrowserView


TEMPLATE = """\
var plonemeeting_delete_meeting_confirm_message = "%(plonemeeting_delete_meeting_confirm_message)s";
var sure_to_remove_selected_items = "%(sure_to_remove_selected_items)s";
var sure_to_present_selected_items = "%(sure_to_present_selected_items)s";
var sure_to_cancel_edit = "%(sure_to_cancel_edit)s";
var are_you_sure = "%(are_you_sure)s";
var sure_to_take_over = "%(sure_to_take_over)s";
var wait_msg = "%(wait_msg)s";
var reinit_delay_confirm_message = "%(reinit_delay_confirm_message)s";
"""


class JSVariables(BrowserView):

    def __call__(self, *args, **kwargs):
        response = self.request.response
        response.setHeader('content-type', 'text/javascript;;charset=utf-8')

        plonemeeting_delete_meeting_confirm_message = translate('plonemeeting_delete_meeting_confirm_message',
                                                                domain='PloneMeeting',
                                                                context=self.request)
        sure_to_remove_selected_items = translate('sure_to_remove_selected_items',
                                                  domain='PloneMeeting',
                                                  context=self.request)
        sure_to_present_selected_items = translate('sure_to_present_selected_items',
                                                   domain='PloneMeeting',
                                                   context=self.request)
        sure_to_cancel_edit = translate('sure_to_cancel_edit',
                                        domain='PloneMeeting',
                                        context=self.request)
        are_you_sure = translate('are_you_sure',
                                 domain='PloneMeeting',
                                 context=self.request)
        sure_to_take_over = translate('sure_to_take_over',
                                      domain='PloneMeeting',
                                      context=self.request)
        wait_msg = translate('wait_msg',
                             domain='PloneMeeting',
                             context=self.request)
        reinit_delay_confirm_message = translate('reinit_delay_confirm_message',
                                                 domain='PloneMeeting',
                                                 context=self.request)

        return TEMPLATE % dict(
            plonemeeting_delete_meeting_confirm_message=plonemeeting_delete_meeting_confirm_message,
            sure_to_remove_selected_items=sure_to_remove_selected_items,
            sure_to_present_selected_items=sure_to_present_selected_items,
            sure_to_cancel_edit=sure_to_cancel_edit,
            are_you_sure=are_you_sure,
            sure_to_take_over=sure_to_take_over,
            wait_msg=wait_msg,
            reinit_delay_confirm_message=reinit_delay_confirm_message,
        )



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from imio.helpers.cache import invalidate_cachekey_volatile_for
from imio.helpers.content import get_vocab
from imio.helpers.content import uuidToObject
from imio.helpers.security import fplog
from persistent.mapping import PersistentMapping
from plone import api
from plone.z3cform.fieldsets.utils import move
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from Products.PloneMeeting.browser.itemassembly import _itemsToUpdate
from Products.PloneMeeting.browser.itemassembly import validate_apply_until_item_number
from Products.PloneMeeting.config import NOT_ENCODED_VOTE_VALUE
from Products.PloneMeeting.config import PMMessageFactory as _
from Products.PloneMeeting.interfaces import IRedirect
from Products.PloneMeeting.utils import _itemNumber_to_storedItemNumber
from Products.PloneMeeting.utils import get_attendee_short_title
from Products.PloneMeeting.utils import notifyModifiedAndReindex
from Products.PloneMeeting.utils import redirect
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.interfaces import NO_VALUE
from zExceptions import BadRequest
from zope import schema
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import provider
from zope.schema._bootstrapinterfaces import IContextAwareDefaultFactory


WRONG_PERSON_UID = "No held_position found with UID \"%s\"!"
WRONG_POSITION_TYPE = "Given position_type \"%s\" does not exist!"


def person_uid_default():
    """
      Get the value from the REQUEST as it is passed when calling the
      form : form?current_delay_row_id=new_value.
    """
    request = getSite().REQUEST
    return request.get('person_uid', u'')


@provider(IContextAwareDefaultFactory)
def apply_until_item_number_default(context):
    """
      Default value is the current item number.
    """
    return safe_unicode(context.getItemNumber(for_display=True))


class IBaseAttendee(Interface):

    person_uid = schema.TextLine(
        title=_(u"Person uid"),
        description=_(u""),
        defaultFactory=person_uid_default,
        required=False)

    apply_until_item_number = schema.TextLine(
        title=_(u"Apply until item number"),
        description=_(u"Specify a number to which this will be applied. "
                      u"Field default is current item number."),
        required=False,
        defaultFactory=apply_until_item_number_default,
        constraint=validate_apply_until_item_number,)


class BaseAttendeeForm(form.Form):
    """Factorize common code used by attendees management forms."""

    description = u''
    _finished = False
    ignoreContext = True  # don't use context to get widget data

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.label = translate(self.label, domain='PloneMeeting', context=request)
        self.request.set('disable_border', 1)
        self.meeting = self.context.getMeeting()

    def updateWidgets(self):
        # XXX manipulate self.fields BEFORE doing form.Form.updateWidgets
        # hide person_uid field
        self.fields['person_uid'].mode = 'hidden'
        move(self, 'apply_until_item_number', after='*')
        form.Form.updateWidgets(self)

    def _update_description(self):
        """Display concerned person as description."""
        person_uid = person_uid_default()
        if person_uid:
            tool = api.portal.get_tool('portal_plonemeeting')
            cfg = tool.getMeetingConfig(self.context)
            hp = uuidToObject(person_uid, unrestricted=True)
            if not hp:
                raise BadRequest(WRONG_PERSON_UID % person_uid)
            self.description = get_attendee_short_title(
                hp, cfg, item=self.context, meeting=self.meeting)

    def update(self):
        """ """
        super(BaseAttendeeForm, self).update()
        self._update_description()
        # after calling parent's update, self.actions are available
        self.actions.get('cancel').addClass('standalone')
        self.buttons = self.buttons.select('apply', 'cancel')

    def extractData(self, setErrors=True):
        """ """
        # when using datagridfield in an overlay, it adds empty rows...
        # removes it
        for wid in self.widgets.values():
            # datagridfield
            if hasattr(wid, 'allow_insert'):
                wid.widgets = [sub_wid for sub_wid in wid.widgets
                               if sub_wid._value != NO_VALUE]
        data, errors = super(BaseAttendeeForm, self).extractData()
        return data, errors

    @button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        # store every data on self
        for k, v in data.items():
            setattr(self, k, v)
        self.apply_until_item_number = \
            _itemNumber_to_storedItemNumber(
                data.get('apply_until_item_number') or u'0'
            )
        self._doApply()
        # in any case, if attendee (un)set absent/excused/... invalidate itemvoters caching
        invalidate_cachekey_volatile_for(
            'Products.PloneMeeting.vocabularies.itemvotersvocabulary',
            get_again=True)
        # invalidate attendees async load on meeting
        invalidate_cachekey_volatile_for(
            'Products.PloneMeeting.browser.async.AsyncLoadMeetingAssemblyAndSignatures',
            get_again=True)

    @button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self._finished = True
        IRedirect(self.request).redirect(self.context.absolute_url())
        return ""

    def mayChangeAttendees(self):
        """ """
        return self.context._mayChangeAttendees()

    def _doApply(self):
        """ """
        if not self.mayChangeAttendees():
            raise Unauthorized

        self.items_to_update = _itemsToUpdate(
            from_item_number=self.context.getItemNumber(relativeTo='meeting'),
            until_item_number=self.apply_until_item_number,
            meeting=self.meeting)

        error = self.mayApplyPrecondition(self.items_to_update)
        if error:
            self._finished = True
            return error

        # a sub form must call
        # error = super(SubAttendeeForm, self)._doApply()
        # if error:
        #     return error
        # then do his job

    def render(self):
        if self._finished:
            # make sure we return nothing, taken into account by ajax query
            return redirect(self.request, self.context.absolute_url())
        return super(BaseAttendeeForm, self).render()

    def _checkMayApplyPrecondition(self, item_to_update):
        """Must return a tuple with error True/False and a msg."""
        error, msg = False, None
        return error, msg

    def mayApplyPrecondition(self, items_to_update):
        """Base method that manage checking if action is authorized.
           A sub form will have to override the _checkMayApplyPrecondition method."""
        error = False
        final_msg = ''
        for item_to_update in items_to_update:
            error, msg = self._checkMayApplyPrecondition(item_to_update)
            if error:
                if item_to_update != self.context:
                    final_msg = translate(
                        "Please check item number ${item_number} at ${item_url}.",
                        mapping={'item_number': item_to_update.getItemNumber(for_display=True),
                                 'item_url': item_to_update.absolute_url()},
                        domain="PloneMeeting", context=self.request)
                    api.portal.show_message(final_msg, type='warning', request=self.request)
                break
        # add a \n before final_msg if any
        if final_msg:
            final_msg = u"\n{0}".format(final_msg)
        return error and (msg + final_msg) or False


class IByeByeAttendee(IBaseAttendee):

    not_present_type = schema.Choice(
        title=_(u"Not present type"),
        description=_(u""),
        vocabulary=u"Products.PloneMeeting.vocabularies.not_present_type_vocabulary",
        required=True)


class ByeByeAttendeeForm(BaseAttendeeForm):
    """ """

    label = _(u'person_byebye')
    schema = IByeByeAttendee
    fields = field.Fields(IByeByeAttendee)

    NOT_PRESENT_MAPPING = {'absent': 'item_absents',
                           'excused': 'item_excused'}

    def _checkMayApplyPrecondition(self, item_to_update):
        """Check that:
           - is an attendee on the meeting;
           - is not a signatory on the item;
           - is an attendee on the item;
           - is not a voter."""
        error = False
        msg = None
        # attendee not present on meeting
        if self.person_uid not in self.meeting.get_attendees():
            msg = translate(
                "Can not set \"${not_present_type}\" a person that is not present on the meeting!",
                mapping={'not_present_type': _(self.not_present_type)},
                domain="PloneMeeting", context=self.request)
            api.portal.show_message(msg, type='warning', request=self.request)
            error = True
        # item signatory
        elif self.person_uid in item_to_update.get_item_signatories(real=True):
            msg = translate(
                "Can not set \"${not_present_type}\" a person selected as signatory on an item!",
                mapping={'not_present_type': _(self.not_present_type)},
                domain="PloneMeeting", context=self.request)
            api.portal.show_message(msg, type='warning', request=self.request)
            error = True
        # attendee not present on item
        elif self.person_uid not in self.context.get_attendees():
            msg = translate(
                "Can not set \"${not_present_type}\" a person that is not present on the item!",
                mapping={'not_present_type': _(self.not_present_type)},
                domain="PloneMeeting", context=self.request)
            api.portal.show_message(msg, type='warning', request=self.request)
            error = True

        # item voter
        # if not a voter, continue
        if not error and self.context.show_votes():
            voters = item_to_update.get_item_voters()
            if self.person_uid in voters:
                all_item_votes = item_to_update.get_item_votes(
                    ignored_vote_values=[NOT_ENCODED_VOTE_VALUE])
                i = 0
                # used when vote is secret
                len_voters = len(voters)
                for item_vote in all_item_votes:
                    # secret
                    if item_to_update.get_vote_is_secret(self.meeting, vote_number=i):
                        # every voters voted?
                        encoded_votes_count = item_to_update.get_vote_count(
                            self.meeting, vote_value='any_voted', vote_number=i)
                        if len_voters == encoded_votes_count:
                            msg = translate(
                                "Can not set \"${not_present_type}\" "
                                "a person that voted on an item!",
                                domain="PloneMeeting",
                                mapping={
                                    'not_present_type':
                                    translate(
                                        self.not_present_type,
                                        domain="PloneMeeting",
                                        context=self.request)},
                                context=self.request)
                            api.portal.show_message(msg,
                                                    type='warning',
                                                    request=self.request)
                            error = True
                    # public
                    else:
                        if self.person_uid in item_vote['voters']:
                            msg = translate("Can not set \"${not_present_type}\" "
                                            "a person that voted on an item!",
                                            domain="PloneMeeting",
                                            mapping={
                                                'not_present_type':
                                                translate(
                                                    self.not_present_type,
                                                    domain="PloneMeeting",
                                                    context=self.request)},
                                            context=self.request)
                            api.portal.show_message(msg,
                                                    type='warning',
                                                    request=self.request)
                            error = True
        return error, msg

    def _doApply(self):
        """ """
        error = super(ByeByeAttendeeForm, self)._doApply()
        if error:
            return error

        # apply item_absents/item_excused
        meeting_not_present_attr = getattr(
            self.meeting, self.NOT_PRESENT_MAPPING[self.not_present_type])
        for item_to_update in self.items_to_update:
            item_to_update_uid = item_to_update.UID()
            item_not_present = meeting_not_present_attr.get(item_to_update_uid, [])
            if self.person_uid not in item_not_present:
                item_not_present.append(self.person_uid)
                meeting_not_present_attr[item_to_update_uid] = item_not_present
                notifyModifiedAndReindex(item_to_update)
        first_item_number = self.items_to_update[0].getItemNumber(for_display=True)
        last_item_number = self.items_to_update[-1].getItemNumber(for_display=True)
        extras = 'item={0} hp={1} not_present_type={2} from_item_number={3} until_item_number={4}'.format(
            repr(self.context), self.person_uid, self.not_present_type, first_item_number, last_item_number)
        fplog('byebye_item_attendee', extras=extras)
        api.portal.show_message(
            _("Attendee has been set ${not_present_type}.",
              mapping={'not_present_type': _('{0}'.format(self.not_present_type))}),
            request=self.request)
        self._finished = True


# do not wrap_form or it breaks the portal_messages displayed in xhr request
# ByeByeAttendeeFormWrapper = wrap_form(ByeByeAttendeeForm)


class IWelcomeAttendee(IBaseAttendee):
    """ """


class WelcomeAttendeeForm(BaseAttendeeForm):
    """ """

    label = _(u"person_welcome")
    schema = IWelcomeAttendee
    fields = field.Fields(IWelcomeAttendee)
    attendee_welcome_msg = _("Attendee has been set back present.")

    def _get_meeting_absent_attr(self):
        """ """
        attr = None
        if self.person_uid in self.context.get_item_absents():
            attr = self.meeting.item_absents
        elif self.person_uid in self.context.get_item_excused():
            attr = self.meeting.item_excused
        else:
            attr = self.meeting.item_non_attendees
        return attr

    def _doApply(self):
        """ """
        error = super(WelcomeAttendeeForm, self)._doApply()
        if error:
            return error

        # check where is person_uid, item_absents/item_excused/item_non_attendees
        meeting_absent_attr = self._get_meeting_absent_attr()
        for item_to_update in self.items_to_update:
            item_to_update_uid = item_to_update.UID()
            item_absents = meeting_absent_attr.get(item_to_update_uid, [])
            if self.person_uid in item_absents:
                item_absents.remove(self.person_uid)
                meeting_absent_attr[item_to_update_uid] = item_absents
                notifyModifiedAndReindex(item_to_update)
        first_item_number = self.items_to_update[0].getItemNumber(for_display=True)
        last_item_number = self.items_to_update[-1].getItemNumber(for_display=True)
        extras = 'item={0} hp={1} from_item_number={2} until_item_number={3}'.format(
            repr(self.context), self.person_uid, first_item_number, last_item_number)
        fplog('welcome_item_attendee', extras=extras)
        api.portal.show_message(self.attendee_welcome_msg, request=self.request)
        self._finished = True


# do not wrap_form or it breaks the portal_messages displayed in xhr request
# WelcomeAttendeeFormWrapper = wrap_form(WelcomeAttendeeForm)


class IByeByeNonAttendee(IBaseAttendee):
    """ """


class ByeByeNonAttendeeForm(ByeByeAttendeeForm):
    """ """

    label = _(u'nonattendee_byebye')
    schema = IByeByeNonAttendee
    fields = field.Fields(IByeByeNonAttendee)
    NOT_PRESENT_MAPPING = {'non_attendee': 'item_non_attendees'}
    not_present_type = 'non_attendee'


# do not wrap_form or it breaks the portal_messages displayed in xhr request
# ByeByeNonAttendeeFormWrapper = wrap_form(ByeByeNonAttendeeForm)


class IWelcomeNonAttendee(IBaseAttendee):
    """ """


class WelcomeNonAttendeeForm(WelcomeAttendeeForm):
    """ """

    label = _(u"nonattendee_welcome")
    schema = IWelcomeNonAttendee
    fields = field.Fields(IWelcomeNonAttendee)
    attendee_welcome_msg = _("Attendee has been set back attendee.")

    def _get_meeting_absent_attr(self):
        """ """
        return self.meeting.item_non_attendees


# do not wrap_form or it breaks the portal_messages displayed in xhr request
# WelcomeNonAttendeeFormWrapper = wrap_form(WelcomeNonAttendeeForm)


def position_type_default():
    """ """
    person_uid = person_uid_default()
    hp = uuidToObject(person_uid, unrestricted=True)
    position_type = hp.secondary_position_type or hp.position_type
    return position_type


def signature_number_default():
    """
      Get the value from the REQUEST as it is passed when editing
      already redefined item signatory.
    """
    request = getSite().REQUEST
    return request.get('signature_number', u'1')


class IRedefineSignatory(IBaseAttendee):

    position_type = schema.Choice(
        title=_(u"Signature position type"),
        description=_(u"Position type to use as label for the signature."),
        defaultFactory=position_type_default,
        required=True,
        vocabulary="PMPositionTypes")

    signature_number = schema.Choice(
        title=_(u"Signature number"),
        description=_(u""),
        defaultFactory=signature_number_default,
        required=True,
        vocabulary=u"Products.PloneMeeting.vocabularies.numbersvocabulary")


def _set_meeting_item_signatory(meeting, item_uid, signature_number, hp_uid, position_type):
    """Set an item signatory.  If already item signatory, it may be directly
       changed to another signatory number."""
    # check if already redefined, if already redefined, remove it
    item_sigantories = meeting.item_signatories.get(item_uid, {})
    for item_signature_number, item_signatory in item_sigantories.items():
        if item_signatory['hp_uid'] == hp_uid:
            # remove redefined item signatory
            _remove_item_signatory(meeting, item_uid, item_signature_number)

    item_signatories = meeting.item_signatories.get(item_uid, PersistentMapping())
    item_signatories[signature_number] = PersistentMapping(
        {'hp_uid': hp_uid, 'position_type': position_type})
    meeting.item_signatories[item_uid] = item_signatories


class RedefineSignatoryForm(BaseAttendeeForm):
    """ """

    label = _(u'Define this attendee as signatory for this item')
    schema = IRedefineSignatory
    fields = field.Fields(IRedefineSignatory)

    def _checkMayApplyPrecondition(self, item_to_update):
        """Check that person_uid:
           - is not already a signatory, on meeting or item;
           - is present;
           - position_type exists."""
        error = False
        msg = None
        # these checks are essentially for restapi as Web UI will prevent these actions
        if self.person_uid in self.meeting.get_signatories():
            msg = translate(
                "Can not set \"Signatory\" a person that is already signatory on the meeting!",
                domain="PloneMeeting", context=self.request)
            api.portal.show_message(msg, type='warning', request=self.request)
            error = True
        elif self.person_uid not in self.meeting.get_attendees():
            msg = translate(
                "Can not set \"Signatory\" a person that is not present on the meeting!",
                domain="PloneMeeting", context=self.request)
            api.portal.show_message(msg, type='warning', request=self.request)
            error = True
        elif self.position_type not in get_vocab(self.context, self.schema['position_type'].vocabularyName):
            raise BadRequest(WRONG_POSITION_TYPE % self.position_type)
        return error, msg

    def _doApply(self):
        """ """
        error = super(RedefineSignatoryForm, self)._doApply()
        if error:
            return error

        # apply signatory
        for item_to_update in self.items_to_update:
            item_to_update_uid = item_to_update.UID()
            _set_meeting_item_signatory(
                self.meeting,
                item_to_update_uid,
                self.signature_number,
                self.person_uid,
                self.position_type)
            notifyModifiedAndReindex(item_to_update)
        first_item_number = self.items_to_update[0].getItemNumber(for_display=True)
        last_item_number = self.items_to_update[-1].getItemNumber(for_display=True)
        extras = 'item={0} hp={1} signature_number={2} from_item_number={3} until_item_number={4}'.format(
            repr(self.context), self.person_uid, self.signature_number, first_item_number, last_item_number)
        fplog('redefine_item_signatory', extras=extras)
        api.portal.show_message(_("Attendee has been set signatory."), request=self.request)
        self._finished = True


# do not wrap_form or it breaks the portal_messages displayed in xhr request
# RedefinedSignatoryFormWrapper = wrap_form(RedefinedSignatoryForm)


class IRemoveRedefinedSignatory(IBaseAttendee):
    """ """


def _remove_item_signatory(meeting, item_uid, signature_number):
    """ """
    del meeting.item_signatories[item_uid][signature_number]
    # if no more redefined item signatories,
    # remove item UID from meeting.item_signatories
    if not meeting.item_signatories[item_uid]:
        del meeting.item_signatories[item_uid]


class RemoveRedefinedSignatoryForm(BaseAttendeeForm):
    """ """

    label = _(u'Remove attendee from signatory defined for this item')
    schema = IRemoveRedefinedSignatory
    fields = field.Fields(IRemoveRedefinedSignatory)

    def _doApply(self):
        """ """
        error = super(RemoveRedefinedSignatoryForm, self)._doApply()
        if error:
            return error

        # apply signatory
        for item_to_update in self.items_to_update:
            item_to_update_uid = item_to_update.UID()
            item_signatories = self.meeting.item_signatories.get(item_to_update_uid, {})
            signature_number = [k for k, v in item_signatories.items()
                                if v['hp_uid'] == self.person_uid]
            if signature_number:
                _remove_item_signatory(self.meeting, item_to_update_uid, signature_number[0])
                notifyModifiedAndReindex(item_to_update)
        first_item_number = self.items_to_update[0].getItemNumber(for_display=True)
        last_item_number = self.items_to_update[-1].getItemNumber(for_display=True)
        extras = 'item={0} hp={1} from_item_number={2} until_item_number={3}'.format(
            repr(self.context), self.person_uid, first_item_number, last_item_number)
        fplog('remove_redefined_item_signatory', extras=extras)
        api.portal.show_message(
            _("Attendee is no more defined as item signatory."), request=self.request)
        self._finished = True


# do not wrap_form or it breaks the portal_messages displayed in xhr request
# RemoveRedefinedSignatoryFormWrapper = wrap_form(RemoveRedefinedSignatoryForm)


class IRedefineAttendeePosition(IBaseAttendee):

    position_type = schema.Choice(
        title=_(u"Position type to use"),
        description=_(u"Position type to use for the attendee on this item."),
        defaultFactory=position_type_default,
        required=True,
        vocabulary="PMAttendeeRedefinePositionTypes")


def set_meeting_item_attendee_position(meeting, item_uid, hp_uid, position_type):
    """ """
    updated = False
    item_attendees_positions = meeting.item_attendees_positions.get(item_uid, PersistentMapping())
    if hp_uid not in item_attendees_positions.values():
        updated = True
        item_attendees_positions[hp_uid] = PersistentMapping(
            {'position_type': position_type})
        meeting.item_attendees_positions[item_uid] = item_attendees_positions
    return updated


class RedefineAttendeePositionForm(BaseAttendeeForm):
    """ """

    label = _(u'Redefine position for this attendee for this item')
    schema = IRedefineAttendeePosition
    fields = field.Fields(IRedefineAttendeePosition)

    def _checkMayApplyPrecondition(self, item_to_update):
        """Condition to apply:
           - new position_type must exist."""
        error = False
        msg = None
        if self.position_type not in get_vocab(self.context, "PMAttendeeRedefinePositionTypes"):
            # could only happen with restapi
            raise BadRequest(WRONG_POSITION_TYPE % self.position_type)
        return error, msg

    def _doApply(self):
        """ """
        error = super(RedefineAttendeePositionForm, self)._doApply()
        if error:
            return error

        # apply redefined position
        for item_to_update in self.items_to_update:
            item_to_update_uid = item_to_update.UID()
            updated = set_meeting_item_attendee_position(
                self.meeting,
                item_to_update_uid,
                self.person_uid,
                self.position_type)
            if updated:
                notifyModifiedAndReindex(item_to_update)
        first_item_number = self.items_to_update[0].getItemNumber(for_display=True)
        last_item_number = self.items_to_update[-1].getItemNumber(for_display=True)
        extras = 'item={0} hp={1} from_item_number={2} until_item_number={3}'.format(
            repr(self.context), self.person_uid, first_item_number, last_item_number)
        fplog('redefine_item_attendee_position', extras=extras)
        api.portal.show_message(
            _("Attendee position has been redefined."), request=self.request)
        self._finished = True


class IRemoveRedefinedAttendeePosition(IBaseAttendee):
    """ """


class RemoveRedefinedAttendeePositionForm(BaseAttendeeForm):
    """ """

    label = _(u'Remove redefined attendee position for this item')
    schema = IRemoveRedefinedAttendeePosition
    fields = field.Fields(IRemoveRedefinedAttendeePosition)

    def _doApply(self):
        """ """
        error = super(RemoveRedefinedAttendeePositionForm, self)._doApply()
        if error:
            return error

        # apply removal of redefined attendee position
        for item_to_update in self.items_to_update:
            item_to_update_uid = item_to_update.UID()
            item_attendees_positions = self.meeting.item_attendees_positions.get(
                item_to_update_uid, {})
            if self.person_uid in item_attendees_positions:
                del item_attendees_positions[self.person_uid]
            if not item_attendees_positions:
                del self.meeting.item_attendees_positions[item_to_update_uid]
                notifyModifiedAndReindex(item_to_update)
        first_item_number = self.items_to_update[0].getItemNumber(for_display=True)
        last_item_number = self.items_to_update[-1].getItemNumber(for_display=True)
        extras = 'item={0} hp={1} from_item_number={2} until_item_number={3}'.format(
            repr(self.context), self.person_uid, first_item_number, last_item_number)
        fplog('remove_redefined_item_attendee_position', extras=extras)
        api.portal.show_message(
            _("Redefined attendee position was removed."), request=self.request)
        self._finished = True


class ChangeAttendeeOrderView(BrowserView):
    """ """

    def __call__(self, attendee_uid, position):
        """ """
        if not self.context._mayChangeAttendees():
            raise Unauthorized
        # get attendee and move it to right position
        meeting = self.context.getMeeting()
        all_uids = list(self.context.get_all_attendees(the_objects=False))
        attendee_uid_index = all_uids.index(attendee_uid)
        all_uids.insert(position - 1, all_uids.pop(attendee_uid_index))
        all_uids = tuple(all_uids)
        context_uid = self.context.UID()
        # if finally the order is back to the order of the meeting
        # remove the item UID from item_attendees_order
        if all_uids == meeting.get_all_attendees(the_objects=False):
            meeting.item_attendees_order.pop(context_uid)
        else:
            meeting._set_item_attendees_order(context_uid, all_uids)

        # log
        extras = 'item={0} hp={1} position={2}'.format(
            repr(self.context), attendee_uid, position)
        fplog('change_item_attendees_order', extras=extras)
        # message
        api.portal.show_message(
            _("Attendee position was changed."), request=self.request)
        return True


class ReinitAttendeesOrderView(BrowserView):
    """ """

    def __call__(self):
        """ """
        if not self.context._mayChangeAttendees():
            raise Unauthorized

        meeting = self.context.getMeeting()
        meeting.item_attendees_order.pop(self.context.UID())

        # log
        extras = 'item={0}'.format(repr(self.context))
        fplog('reinit_item_attendees_order', extras=extras)
        # message
        api.portal.show_message(
            _("Attendees order was reinitialized to meeting order for this item."),
            request=self.request)
        return redirect(self.request, self.context.absolute_url())



# -*- coding: utf-8 -*-

from plone import api
from Products.cron4plone.browser.configlets.cron_configuration import ICronConfiguration
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from zope.component import queryUtility


class Migrate_To_4107(Migrator):

    def _moveToItemCreatedOnlyUsingTemplate(self):
        """Empty item are now also created using an itemTemplate, move to it."""
        logger.info("Moving to empty item created from an item template...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            default_template = cfg._create_default_item_template()
            if default_template:
                # move it to the top
                folder = default_template.aq_inner.aq_parent
                folder.folder_position_typeaware(position='top', id=default_template.getId())
                # if cfg.itemCreatedOnlyUsingTemplate was True, disable created default template
                if getattr(cfg, 'itemCreatedOnlyUsingTemplate', False) is True:
                    api.content.transition(default_template, transition='deactivate')
        # remove useless MeetingConfig.itemCreatedOnlyUsingTemplate field
        self.cleanMeetingConfigs(field_names=['itemCreatedOnlyUsingTemplate'])
        logger.info('Done.')

    def _registerMeetingFactoryTypes(self):
        """Make sure every relevant portal_types are correctly registered in portal_factory."""
        logger.info("Registering every Meeting types into portal_factory...")
        self.ps.runImportStepFromProfile('profile-Products.PloneMeeting:default', 'factorytool')
        # This will register portal_types to portal_factory
        self.reloadMeetingConfigs()
        logger.info('Done.')

    def _updateUpdateDelayAwareAdvicecsCronJobTime(self):
        """Set @@update-delay-aware-advices cronjob time to 01:45."""
        logger.info("Setting cronjob for @@update-delay-aware-advices to 01:45...")
        cron_configlet = queryUtility(ICronConfiguration, 'cron4plone_config')
        cron_configlet.cronjobs = [u'45 1 * * portal/@@update-delay-aware-advices']
        logger.info('Done.')

    def run(self, from_migration_to_41=False):
        logger.info('Migrating to PloneMeeting 4107...')
        self._moveToItemCreatedOnlyUsingTemplate()
        self._registerMeetingFactoryTypes()
        self._updateUpdateDelayAwareAdvicecsCronJobTime()
        # reapply typeinfo as imio.zamqp.pm typeinfo was merged into Products.PloneMeeting
        self.ps.runImportStepFromProfile('profile-Products.PloneMeeting:default', 'typeinfo')


def migrate(context):
    '''This migration function will:

       1) Remove field 'itemCreatedOnlyUsingTemplate' from every MeetingConfigs;
       2) Make sure every relevant portal_types are correctly registered in portal_factory;
       3) Set @@update-delay-aware-advices cronjob time to 01:45;
       4) Re-apply typeinfo to include imio.zamqp.pm informations.
    '''
    migrator = Migrate_To_4107(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-

from eea.facetednavigation.interfaces import ICriteria
from Products.Archetypes.event import ObjectEditedEvent
from Products.CMFPlone.interfaces.constrains import IConstrainTypes
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from zope.event import notify


class Migrate_To_4202(Migrator):

    def _updateFacetedFilters(self):
        """Update vocabulary used for "Taken over by"."""
        logger.info("Updating faceted filter \"Taken over by\" for every MeetingConfigs...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            criteria = ICriteria(cfg.searches.searches_items)
            criteria.edit(
                'c12', **{
                    'vocabulary':
                        'Products.PloneMeeting.vocabularies.creatorswithnobodyforfacetedfiltervocabulary'})
        logger.info('Done.')

    def _fixPreAcceptedWFA(self):
        """Update MeetingConfigs using the 'pre_accepted' WFA and
           update workflow mappings for items in state 'pre_accepted'."""
        logger.info('Updating "pre_accepted" WFAdaptation for every MeetingConfigs...')
        for cfg in self.tool.objectValues('MeetingConfig'):
            if 'pre_accepted' in cfg.getWorkflowAdaptations():
                brains = self.catalog(portal_type=cfg.getItemTypeName(), review_state='pre_accepted')
                logger.info('The "pre_accepted" WFAdaptation was found for "{0}", '
                            'updating "{1}" items...'.format(cfg.getId(), len(brains)))
                notify(ObjectEditedEvent(cfg))
                item_wf = cfg.getItemWorkflow(True)
                for brain in brains:
                    item = brain.getObject()
                    changed = item_wf.updateRoleMappingsFor(item)
                    if changed:
                        item.reindexObjectSecurity()
        logger.info('Done.')

    def _fixFacetedFoldersConstrainTypes(self):
        """Fix constrainTypes for every faceted folder stored in user personnal folder."""
        logger.info('Updating searches_... folders constraintypes for every users...')
        for cfg in self.tool.objectValues('MeetingConfig'):
            folders = cfg._get_all_meeting_folders()
            for folder in folders:
                sub_folders = [sub_folder for sub_folder in folder.objectValues('ATFolder')
                               if sub_folder.getId().startswith('searches_')]
                for sub_folder in sub_folders:
                    constrain = IConstrainTypes(sub_folder)
                    constrain.setConstrainTypesMode(1)
                    allowedTypes = []
                    constrain.setLocallyAllowedTypes(allowedTypes)
                    constrain.setImmediatelyAddableTypes(allowedTypes)
        logger.info('Done.')

    def run(self, extra_omitted=[], from_migration_to_4200=False):

        logger.info('Migrating to PloneMeeting 4202...')

        # not necessary if executing the full upgrade to 4200
        if not from_migration_to_4200:
            self._updateFacetedFilters()
            self._fixPreAcceptedWFA()

        # this is not managed by the main upgrade to 4200
        self._fixFacetedFoldersConstrainTypes()


def migrate(context):
    '''This migration function will:

       1) Update the vocabulary used by the "c12" faceted filter (Taken over by);
       2) Fix the MeetingConfigs and items using the 'pre_accepted' WFAdaptation;
       3) Fix constrainTypes for every faceted folder of every users.
    '''
    migrator = Migrate_To_4202(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from imio.helpers.content import get_user_fullname
from imio.history.interfaces import IImioHistory
from imio.history.utils import add_event_to_history
from plone import api
from Products.Archetypes import DisplayList
from Products.Five.browser import BrowserView
from Products.PloneMeeting.config import PMMessageFactory as _
from zope.component import getAdapter


class ItemCompletenessView(BrowserView):
    '''Render the item completeness HTML on the meetingitem_view.'''

    def __init__(self, context, request):
        super(BrowserView, self).__init__(context, request)
        self.context = context
        self.request = request
        self.portal_url = api.portal.get().absolute_url()

    def listSelectableCompleteness(self):
        '''Returns a list of completeness the current user can set the item to.'''
        # we base our actions on avaialble terms in the MeetingItem.completeness field vocabulary
        completenesses = self.context.listCompleteness()
        completenessKeys = completenesses.keys()
        currentCompleteness = self.context.getCompleteness()
        # now check if user can evaluate completeness
        if not self.context.adapted().mayEvaluateCompleteness():
            completenessKeys.remove('completeness_complete')
            completenessKeys.remove('completeness_incomplete')
            completenessKeys.remove('completeness_not_yet_evaluated')
            completenessKeys.remove('completeness_evaluation_not_required')
        # now check if user can set to 'completeness_evaluation_asked_again'
        if not self.context.adapted().mayAskCompletenessEvalAgain():
            completenessKeys.remove('completeness_evaluation_asked_again')
        # now if currentComplenteness is still in completeness, we remove it
        if currentCompleteness in completenessKeys:
            completenessKeys.remove(currentCompleteness)
        # now build a vocabulary with left values
        res = []
        for completeness in completenesses.items():
            if completeness[0] in completenessKeys:
                res.append(completeness)
        return DisplayList(tuple(res))


class ChangeItemCompletenessView(BrowserView):
    '''This manage the overlay popup displayed to enter a comment when the completeness is changed.'''

    def __init__(self, context, request):
        super(BrowserView, self).__init__(context, request)
        self.context = context
        self.request = request

    def __call__(self):
        form = self.request.form
        submitted = form.get('form.buttons.save', False) or form.get('form.submitted', False)
        cancelled = form.get('form.buttons.cancel', False)
        if cancelled:
            # the only way to enter here is the popup overlay not to be shown
            # because while using the popup overlay, the jQ function take care of hidding it
            # while the Cancel button is hit
            return self.request.RESPONSE.redirect(self.context.absolute_url())
        elif submitted:
            # change completeness value, it will also check that given 'new_completeness_value'
            # is available in the field vocabulary if not available, and selectable by current user
            self._changeCompleteness(self.request.get('new_completeness_value'),
                                     comment=self.request.get('comment', ''))
            return self.request.RESPONSE.redirect(self.context.absolute_url())
        return self.index()

    def _changeCompleteness(self, new_completeness_value, bypassSecurityCheck=False, comment=''):
        '''Helper method that change completeness and manage completeness history.'''
        # make sure new_completeness_value exists in MeetingItem.listCompleteness vocabulary
        if new_completeness_value not in self.context.listCompleteness():
            raise KeyError("New value %s does not correspond to a value of MeetingItem.listCompleteness")

        if not bypassSecurityCheck and new_completeness_value not in \
           self.context.unrestrictedTraverse('@@item-completeness').listSelectableCompleteness():
            raise Unauthorized
        self.context.setCompleteness(new_completeness_value)
        # add a line to the item's completeness_changes_history
        add_event_to_history(
            self.context,
            'completeness_changes_history',
            action=new_completeness_value,
            comments=comment)
        self.context._update_after_edit(idxs=['getCompleteness'])
        plone_utils = api.portal.get_tool('plone_utils')
        plone_utils.addPortalMessage(_("Item completeness changed."))


class ItemCompletenessHistoryView(BrowserView):
    '''Display history of emergency value changes.'''
    def __init__(self, context, request):
        super(BrowserView, self).__init__(context, request)
        self.context = context
        self.request = request
        self.tool = api.portal.get_tool('portal_plonemeeting')

    def getHistory(self, checkMayViewEvent=True, checkMayViewComment=True):
        """ """
        adapter = getAdapter(self.context, IImioHistory, 'completeness_changes')
        history = adapter.getHistory(
            checkMayViewEvent=checkMayViewEvent,
            checkMayViewComment=checkMayViewComment)
        if not history:
            return []
        history.sort(key=lambda x: x["time"], reverse=True)
        return history

    def renderComments(self, comments, mimetype='text/plain'):
        """
          Borrowed from imio.history.
        """
        transformsTool = api.portal.get_tool('portal_transforms')
        data = transformsTool.convertTo('text/x-html-safe', comments, mimetype=mimetype)
        return data.getData()

    def get_user_fullname(self, user_id):
        return get_user_fullname(user_id)



# -*- coding: utf-8 -*-

from collective.contact.plonegroup.browser.settings import IContactPlonegroupConfig
from collective.contact.plonegroup.config import get_registry_functions
from collective.contact.plonegroup.config import get_registry_organizations
from collective.contact.plonegroup.config import set_registry_functions
from collective.contact.plonegroup.utils import get_organizations
from collective.contact.plonegroup.utils import get_plone_group
from copy import deepcopy
from plone import api
from Products.PloneMeeting.config import PMMessageFactory as _
from Products.PloneMeeting.tests.PloneMeetingTestCase import PloneMeetingTestCase
from Products.PloneMeeting.validators import PloneGroupSettingsFunctionsValidator
from Products.PloneMeeting.validators import PloneGroupSettingsOrganizationsValidator
from Products.validation import validation
from zope.i18n import translate
from zope.interface import Invalid


class testValidators(PloneMeetingTestCase):
    '''Tests the validators.'''

    def test_pm_IsValidCertifiedSignaturesValidatorWorking(self):
        '''Test the 'isCertifiedSignaturesValidator' validator.
           Here we are testing that working cases are actually working..."""
           It fails if :
           - signatures are not ordered by signature number;
           - both date_from/date_to are not provided together (if provided);
           - date format is wrong (respect YYYY/DD/MM, valid DateTime, date_from <= date_to).'''
        v = validation.validatorFor('isValidCertifiedSignatures')
        # if nothing is defined, validation is successful
        self.failIf(v([]))
        # nothing is required, except signatureNumber
        # here is a working sample
        certified = [
            {'signatureNumber': '1',
             'name': 'Name1a',
             'function': 'Function1a',
             'date_from': '2015/01/01',
             'date_to': '2015/02/02',
             },
            {'signatureNumber': '1',
             'name': 'Name1b',
             'function': 'Function1b',
             'date_from': '2015/02/15',
             'date_to': '2015/02/15',
             },
            {'signatureNumber': '1',
             'name': 'Name1',
             'function': 'Function1',
             'date_from': '',
             'date_to': '',
             },
            {'signatureNumber': '2',
             'name': 'Name2a',
             'function': 'Function2a',
             'date_from': '2015/01/01',
             'date_to': '2015/01/15',
             },
            {'signatureNumber': '2',
             'name': 'Name2',
             'function': 'Function2',
             'date_from': '',
             'date_to': '',
             },
        ]
        self.failIf(v(certified))

        # every signatures are not mandatorily redefined
        # this is the case especially while overriding for example signature 2
        # on a MeetingGroup and keep signature 1 from the MeetingConfig
        # fails if signatureNumber are not ordered
        certified = [
            {'signatureNumber': '2',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
        ]
        self.failIf(v(certified))

    def test_pm_IsValidCertifiedSignaturesValidatorFailsIfNotOrdered(self):
        '''Test the 'isCertifiedSignaturesValidator' validator.
           It fails if signatures are not ordered by signature number.'''
        v = validation.validatorFor('isValidCertifiedSignatures')
        # fails if signatureNumber are not ordered
        certified = [
            {'signatureNumber': '2',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
        ]
        order_error_msg = translate('error_certified_signatures_order',
                                    domain='PloneMeeting',
                                    context=self.portal.REQUEST)
        self.assertEquals(v(certified),
                          order_error_msg)

    def test_pm_IsValidCertifiedSignaturesValidatorFailsIfBothDatesNotProvided(self):
        '''Test the 'isCertifiedSignaturesValidator' validator.
           It fails if signatures both dates are not provided.'''
        v = validation.validatorFor('isValidCertifiedSignatures')
        # if we want to use date, we have to provide both date_from and date_to
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '2015/01/01',
             'date_to': '',
             },
        ]
        both_error_msg = translate('error_certified_signatures_both_dates_required',
                                   mapping={'row_number': 1},
                                   domain='PloneMeeting',
                                   context=self.portal.REQUEST)
        self.assertEquals(v(certified),
                          both_error_msg)
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '2015/01/01',
             },
        ]
        self.assertEquals(v(certified),
                          both_error_msg)

    def test_pm_IsValidCertifiedSignaturesValidatorFailsIfWrongDateFormat(self):
        '''Test the 'isCertifiedSignaturesValidator' validator.
           It fails if signatures use wrong format for dates.'''
        v = validation.validatorFor('isValidCertifiedSignatures')
        # check date format
        # wrong date
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '2015/02/00',
             'date_to': '2015/02/15',
             },
        ]
        invalid_dates_error_msg = translate('error_certified_signatures_invalid_dates',
                                            mapping={'row_number': 1},
                                            domain='PloneMeeting',
                                            context=self.portal.REQUEST)
        self.assertEquals(v(certified),
                          invalid_dates_error_msg)
        # wrong date format, not respecting YYYY/MM/DD
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '2015/15/01',
             'date_to': '2015/20/01',
             },
        ]
        self.assertEquals(v(certified),
                          invalid_dates_error_msg)
        # date_from must be <= date_to
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '2015/01/15',
             'date_to': '2015/01/10',
             },
        ]
        self.assertEquals(v(certified),
                          invalid_dates_error_msg)

        # row number is displayed in the error message, check that it works
        # here, row 2 is wrong
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': 'wrong_date',
             'date_to': '2015/01/01',
             },
            {'signatureNumber': '2',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
        ]
        invalid_dates_error_msg2 = translate('error_certified_signatures_invalid_dates',
                                             mapping={'row_number': 2},
                                             domain='PloneMeeting',
                                             context=self.portal.REQUEST)
        self.assertEquals(v(certified),
                          invalid_dates_error_msg2)

    def test_pm_IsValidCertifiedSignaturesValidatorFailIfUsingDuplicatedEntries(self):
        '''Test the 'isCertifiedSignaturesValidator' validator.
           It fails if 2 entries use exactly same signatureNumber/date_from/date_to.'''
        v = validation.validatorFor('isValidCertifiedSignatures')
        # test first without dates, row 3 is wrong
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
            {'signatureNumber': '2',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
            {'signatureNumber': '2',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
        ]
        duplicated_entries_error_msg = translate('error_certified_signatures_duplicated_entries',
                                                 mapping={'row_number': 3},
                                                 domain='PloneMeeting',
                                                 context=self.portal.REQUEST)
        self.assertEquals(v(certified),
                          duplicated_entries_error_msg)
        # test with dates, row 2 is wrong
        certified = [
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '2015/01/01',
             'date_to': '2015/02/02',
             },
            {'signatureNumber': '1',
             'name': '',
             'function': '',
             'date_from': '2015/01/01',
             'date_to': '2015/02/02',
             },
            {'signatureNumber': '2',
             'name': '',
             'function': '',
             'date_from': '',
             'date_to': '',
             },
        ]
        duplicated_entries_error_msg2 = translate('error_certified_signatures_duplicated_entries',
                                                  mapping={'row_number': 2},
                                                  domain='PloneMeeting',
                                                  context=self.portal.REQUEST)
        self.assertEquals(v(certified),
                          duplicated_entries_error_msg2)

    def test_pm_PloneGroupSettingsFunctionsValidator(self):
        """Completed plonegroup settings validation with our use cases :
           - can not remove a suffix if used in MeetingConfig.selectableCopyGroups;
           - can not remove a suffix if used in MeetingItem.copyGroups;
           - can not remove a suffix if used as composed value, so like
             'suffix_proposing_group_level1reviewers',
             in MeetingConfig.itemAnnexConfidentialVisibleFor for example;
           - can not remove a suffix used by MeetingConfig.itemWFValidationLevels."""
        # make sure we use default itemWFValidationLevels,
        # useful when test executed with custom profile
        cfg = self.meetingConfig
        self._setUpDefaultItemWFValidationLevels(cfg)

        def _check(validation_error_msg, checks=['without', 'disabled', 'fct_orgs']):
            """ """
            values = []
            if 'without' in checks:
                values.append(functions_without_samplers)
            if 'disabled' in checks:
                values.append(functions_with_disabled_samplers)
            if 'fct_orgs' in checks:
                values.append(functions_with_fct_orgs_samplers)
            for value in values:
                with self.assertRaises(Invalid) as cm:
                    validator.validate(value)
                self.assertEqual(cm.exception.message, validation_error_msg)

        self.changeUser('siteadmin')
        # add a new suffix and play with it
        functions = get_registry_functions()
        functions_without_samplers = deepcopy(functions)
        functions.append({'enabled': True,
                          'fct_management': False,
                          'fct_id': u'samplers',
                          'fct_orgs': [],
                          'fct_title': u'Samplers'})
        functions_with_disabled_samplers = deepcopy(functions)
        functions_with_disabled_samplers[-1]['enabled'] = False
        functions_with_fct_orgs_samplers = deepcopy(functions)
        functions_with_fct_orgs_samplers[-1]['fct_orgs'] = [self.vendors_uid]

        validator = PloneGroupSettingsFunctionsValidator(
            self.portal, self.request, None, IContactPlonegroupConfig['functions'], None)
        self.assertIsNone(validator.validate(functions))
        set_registry_functions(functions)
        # use samplers suffix
        self._enableItemValidationLevel(cfg, level='prevalidated', suffix='samplers')

        # developers_samplers was created
        dev_samplers = get_plone_group(self.developers_uid, 'samplers')
        dev_samplers_id = dev_samplers.getId()
        self.assertTrue(dev_samplers in api.group.get_groups())
        # use samplers in MeetingConfig
        cfg.setSelectableCopyGroups(cfg.getSelectableCopyGroups() + (dev_samplers_id, ))
        validation_error_msg = _('can_not_delete_plone_group_meetingconfig',
                                 mapping={'cfg_url': cfg.absolute_url()})
        _check(validation_error_msg)
        # also check composed values like 'suffix_proposing_group_level1reviewers'
        cfg.setSelectableCopyGroups(())
        cfg.setItemAnnexConfidentialVisibleFor(('suffix_proposing_group_samplers', ))
        _check(validation_error_msg, checks=['without', 'disabled'])
        cfg.setItemAnnexConfidentialVisibleFor(())
        # use samplers on item, remove it from MeetingConfig
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        item.setCopyGroups((dev_samplers_id, ))
        item.reindexObject()
        # still complaining about config because used in itemWFValidationLevels
        _check(validation_error_msg, checks=['without', 'disabled'])
        self._disableItemValidationLevel(cfg, level='prevalidated', suffix='prereviewers')
        validation_error_msg = _('can_not_delete_plone_group_meetingitem',
                                 mapping={'item_url': item.absolute_url()})
        _check(validation_error_msg)
        # remove it on item, then everything is correct
        item.setCopyGroups(())
        item.reindexObject()
        self.assertIsNone(validator.validate(functions))
        set_registry_functions(functions_without_samplers)
        self.assertFalse(dev_samplers in api.group.get_groups())

        # an _advisers may not be disabled if used
        item.setOptionalAdvisers((self.developers_uid, ))
        item._update_after_edit(idxs=['indexAdvisers'])
        functions_with_fct_orgs_advisers = deepcopy(functions)
        self.assertEqual(functions_with_fct_orgs_advisers[0]['fct_id'], u'advisers')
        functions_with_fct_orgs_advisers[0]['fct_orgs'] = [self.vendors_uid]
        with self.assertRaises(Invalid) as cm:
            validator.validate(functions_with_fct_orgs_advisers)
        self.assertEqual(cm.exception.message, validation_error_msg)
        # but if disabling another level it is correct
        # disable level prereviewers for vendors
        functions_with_fct_orgs_prereviewers = deepcopy(functions)
        self.assertEqual(functions_with_fct_orgs_advisers[3]['fct_id'], u'prereviewers')
        functions_with_fct_orgs_prereviewers[3]['fct_orgs'] = [self.vendors_uid]
        self.assertIsNone(validator.validate(functions_with_fct_orgs_prereviewers))
        # remove adviser so it validates
        item.setOptionalAdvisers(())
        item._update_after_edit(idxs=['indexAdvisers'])
        self.assertIsNone(validator.validate(functions_with_fct_orgs_advisers))
        set_registry_functions(functions_with_fct_orgs_advisers)
        self.assertFalse(self.developers_advisers in api.group.get_groups())

    def test_pm_PloneGroupSettingsOrganizationsValidator(self):
        """Can not unselected an organization if used as groups_in_charge
           of another organziations or in MeetingConfig.usingGroups."""
        cfg = self.meetingConfig
        self.changeUser('siteadmin')
        validator = PloneGroupSettingsOrganizationsValidator(
            self.portal, self.request, None, IContactPlonegroupConfig['organizations'], None)
        organizations = get_registry_organizations()
        self.assertIsNone(validator.validate(organizations))

        # org can not be unselected if used in another org.groups_in_charge
        orgs = get_organizations()
        orgs[0].groups_in_charge = [orgs[1].UID()]
        with self.assertRaises(Invalid) as cm:
            validator.validate([organizations[0]])
        validation_error_msg = _('can_not_unselect_plone_group_org',
                                 mapping={'item_url': orgs[0].absolute_url()})
        self.assertEqual(cm.exception.message, validation_error_msg)
        # but other could be unselected
        self.assertIsNone(validator.validate([organizations[1]]))
        # remove groups_in_charge so org may be unselected
        orgs[0].groups_in_charge = []

        # MeetingConfg.usingGroups
        cfg.setUsingGroups([orgs[1].UID()])
        with self.assertRaises(Invalid) as cm:
            validator.validate([organizations[0]])
        validation_error_msg = _('can_not_unselect_plone_group_meetingconfig',
                                 mapping={'cfg_title': cfg.Title()})
        self.assertEqual(cm.exception.message, validation_error_msg)
        # remove usingGroups so org may be unselected
        cfg.setUsingGroups([])

        # now org may be unselected
        self.assertIsNone(validator.validate([organizations[0]]))


def test_suite():
    from unittest import makeSuite
    from unittest import TestSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testValidators, prefix='test_pm_'))
    return suite



# -*- coding: utf-8 -*-
#
# File: Meeting.py
#
# GNU General Public License (GPL)
#

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from appy.gen import No
from plone import api
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.permissions import ReviewPortalContent
from Products.CMFCore.utils import _checkPermission
from Products.PloneMeeting.config import PMMessageFactory as _
from Products.PloneMeeting.content.meeting import Meeting
from Products.PloneMeeting.interfaces import IMeetingWorkflowActions
from Products.PloneMeeting.interfaces import IMeetingWorkflowConditions
from Products.PloneMeeting.utils import fplog
from Products.PloneMeeting.utils import get_annexes
from zope.i18n import translate
from zope.interface import implements

import logging


logger = logging.getLogger('PloneMeeting')


class MeetingWorkflowConditions(object):
    '''Adapts a meeting to interface IMeetingWorkflowConditions.'''
    implements(IMeetingWorkflowConditions)
    security = ClassSecurityInfo()

    def __init__(self, meeting):
        self.context = meeting
        self.tool = api.portal.get_tool('portal_plonemeeting')
        self.cfg = self.tool.getMeetingConfig(self.context)

    security.declarePublic('mayPublish')

    def mayPublish(self):
        if _checkPermission(ReviewPortalContent, self.context):
            return True

    security.declarePublic('mayPublishDecisions')

    def mayPublishDecisions(self):
        '''Used when 'hide_decisions_when_under_writing' wfAdaptation is active.'''
        res = False
        # The user just needs the "Review portal content" permission on the object
        if _checkPermission(ReviewPortalContent, self.context):
            res = True
        return res

    security.declarePublic('mayFreeze')

    def mayFreeze(self):
        res = False
        if _checkPermission(ReviewPortalContent, self.context):
            res = True
        return res

    security.declarePublic('mayDecide')

    def mayDecide(self):
        res = False
        if _checkPermission(ReviewPortalContent, self.context):
            res = True
        return res

    security.declarePublic('mayCorrect')

    def mayCorrect(self, destinationState=None):
        '''See doc in interfaces.py.'''
        if not _checkPermission(ReviewPortalContent, self.context):
            return

        meeting_state = self.context.query_state()
        if meeting_state == 'closed':
            if self.tool.isManager(realManagers=True) or \
               'meetingmanager_correct_closed_meeting' in self.cfg.getWorkflowAdaptations():
                return True
            else:
                return No(_('closed_meeting_not_correctable_by_config'))

        return True

    security.declarePublic('mayClose')

    def mayClose(self):
        if _checkPermission(ReviewPortalContent, self.context):
            return True

    security.declarePublic('may_accept_items')

    def may_accept_items(self):
        if self.context.query_state() in self.cfg.getMeetingStatesAcceptingItemsForMeetingManagers():
            return True

    security.declarePublic('may_change_items_order')

    def may_change_items_order(self):
        res = True
        if not _checkPermission(ModifyPortalContent, self.context) or \
           self.context.query_state() in Meeting.MEETINGCLOSEDSTATES:
            res = False
        return res


InitializeClass(MeetingWorkflowConditions)


class MeetingWorkflowActions(object):
    '''Adapts a meeting to interface IMeetingWorkflowActions.'''
    implements(IMeetingWorkflowActions)
    security = ClassSecurityInfo()

    def __init__(self, meeting):
        self.context = meeting
        self.tool = api.portal.get_tool('portal_plonemeeting')
        self.cfg = self.tool.getMeetingConfig(self.context)

    security.declarePrivate('update_meeting_number')

    def update_meeting_number(self):
        '''When a meeting is published (or frozen, depending on workflow
           adaptations), we attribute him a sequence number.'''
        if not self.context.attribute_is_used('meeting_number') or \
           self.context.meeting_number != -1:
            return  # Not used or already computed.
        prev = self.context.get_previous_meeting(interval=365)
        if "meeting_number" in self.cfg.getYearlyInitMeetingNumbers():
            # I must reinit the meeting number to 1 if it is the first
            # meeting of this year.
            if not prev or \
               (prev.date.year != self.context.date.year):
                self.context.meeting_number = 1
                self.cfg.setLastMeetingNumber(1)
                return
        # If we are here, we must simply increment the meeting number.
        meeting_number = self.cfg.getLastMeetingNumber() + 1
        self.context.meeting_number = meeting_number
        self.cfg.setLastMeetingNumber(meeting_number)
        api.portal.show_message(_("meeting_number_init",
                                  mapping={"meeting_number": meeting_number}),
                                request=self.context.REQUEST)
        # show a warning if previous meeting number is not consistent
        if prev and \
           (prev.date.year == self.context.date.year) and \
           prev.meeting_number != meeting_number - 1:
            api.portal.show_message(
                _("meeting_number_inconsistent",
                  mapping={
                      "previous_meeting_number": prev.meeting_number,
                      "previous_meeting_date": self.tool.format_date(prev.date)}),
                request=self.context.REQUEST, type="warning")

    security.declarePrivate('doPublish')

    def doPublish(self, stateChange):
        '''When publishing the meeting, initialize the sequence number.'''
        self.update_meeting_number()

    security.declarePrivate('doFreeze')

    def doFreeze(self, stateChange):
        '''When freezing the meeting, we initialize sequence number.'''
        self.update_meeting_number()

    security.declarePrivate('doDecide')

    def doDecide(self, stateChange):
        ''' '''
        self.update_meeting_number()
        # Set the firstItemNumber
        self.context.update_first_item_number()

    security.declarePrivate('doClose')

    def doClose(self, stateChange):
        ''' '''
        self.update_meeting_number()
        # Set the firstItemNumber
        self.context.update_first_item_number(force=True)
        # remove annex previews of every items if relevant
        if self.cfg.getRemoveAnnexesPreviewsOnMeetingClosure():
            # add logging message to fingerpointing log
            for item in self.context.get_items(ordered=True):
                annexes = get_annexes(item)
                if annexes:
                    for annex in annexes:
                        # only remove preview if show_preview is 0
                        if item.categorized_elements[annex.UID()]['show_preview'] == 0:
                            self.tool._removeAnnexPreviewFor(item, annex)
                extras = 'item={0} number_of_annexes={1}'.format(repr(item), len(annexes))
                fplog('remove_annex_previews', extras=extras)
            msg = translate(
                u"Preview of annexes were deleted upon meeting closure.",
                domain='PloneMeeting',
                context=self.context.REQUEST)
            api.portal.show_message(msg, request=self.context.REQUEST)

    security.declarePrivate('doPublish_decisions')

    def doPublish_decisions(self, stateChange):
        '''When the wfAdaptation 'hide_decisions_when_under_writing' is activated.'''
        pass

    security.declarePrivate('doBackToCreated')

    def doBackToCreated(self, stateChange):
        ''' '''
        pass

    security.declarePrivate('doBackToDecided')

    def doBackToDecided(self, stateChange):
        # Oups when closing a meeting we have updated the firsItemNumber
        # we need to reverse our action
        self.context.first_item_number = -1

    security.declarePrivate('doBackToPublished')

    def doBackToPublished(self, stateChange):
        ''' '''
        pass

    security.declarePrivate('doBackToDecisionsPublished')

    def doBackToDecisionsPublished(self, stateChange):
        '''When the wfAdaptation 'hide_decisions_when_under_writing' is activated.'''
        pass

    security.declarePrivate('doBackToFrozen')

    def doBackToFrozen(self, stateChange):
        pass


InitializeClass(MeetingWorkflowActions)



# -*- coding: utf-8 -*-

from collections import OrderedDict
from collective.contact.plonegroup.config import ORGANIZATIONS_REGISTRY
from imio.helpers.security import check_zope_admin
from imio.pyutils import system
from plone import api
from plone.app.uuid.utils import uuidToObject
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.utils import safe_unicode
from z3c.relationfield.relation import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import modified

import copy
import os
import phonenumbers


try:
    from collective.behavior.internalnumber.behavior import IInternalNumberBehavior
except ImportError:
    pass


def safe_encode(value, encoding='utf-8'):
    """
        Converts a value to encoding, even if it is already encoded.
    """
    if isinstance(value, unicode):
        return value.encode(encoding)
    return value


def get_organizations(self, obj=False):
    registry = getUtility(IRegistry)
    terms = []
    for uid in registry[ORGANIZATIONS_REGISTRY]:
        title = uuidToObject(uid).get_full_title(separator=' - ', first_index=1)
        terms.append((uid, title))
    if obj:
        return terms
    return '\n'.join(['%s;%s' % (t[0], t[1]) for t in terms])


def import_contacts(self, dochange=True, ownorg='Mon organisation', only='ORGS|PERS|HP', path=''):
    """
        Import contacts from several files in 'Extensions'
        * organizations.csv:    ID;ID Parent;Intitulé;Acronyme;Description;Type;Adr par;Rue;Numéro;Comp adr;
                                CP;Localité;Tél;Gsm;Fax;Courriel;Site;Région;Pays;UID
        * persons.csv:  ID;Nom;Prénom;Genre;Civilité;Naissance;Adr par;Rue;Numéro;Comp adr;
                        CP;Localité;Tél;Gsm;Fax;Courriel;Site;Région;Pays;Num int;UID
        * positions.csv:    ID;ID org;Intitulé;Description;Type;Adr par;Rue;Numéro;Comp adr;
                            CP;Localité;Tél;Gsm;Fax;Courriel;Site;Région;Pays;UID
        * heldpositions.csv:    ID;ID person;ID org;ID fct;Intitulé fct;Début fct;Fin fct;Adr par;Rue;
                                Numéro;Comp adr;CP;Localité;Tél;Gsm;Fax;Courriel;Site;Région;Pays;UID
    """
    if not check_zope_admin():
        return "You must be a zope manager to run this script"
    doit = False
    if dochange not in ('', '0', 'False', 'false'):
        doit = True
    portal = api.portal.get()
    contacts = portal['contacts']

    def digit(phone):
        # filter with str.isdigit or unicode.isdigit
        return filter(type(phone).isdigit, phone)

    def is_zip(zipc, line, typ, country):
        ozipc = zipc
        zipc = digit(zipc)
        if ozipc != zipc:
            out.append("!! %s: line %d, zip code contains non digit chars: %s" % (typ, line, zipc))
        if zipc and len(zipc) != 4 and not country:
            out.append("!! %s: line %d, zip code length not 4: %s" % (typ, line, zipc))
        if zipc in ['0']:
            return ''
        return zipc

    def check_phone(phone, line, typ, country):
        if not phone:
            return phone
        country = country.lower()
        countries = {'belgique': 'BE', 'france': 'FR'}
        if not country:
            ctry = 'BE'
        elif country in countries:
            ctry = countries[country]
        else:
            out.append("!! %s: line %d, country not detected '%s', phone number: '%s'" % (typ, line, country, phone))
            return phone
        try:
            number = phonenumbers.parse(phone, ctry)
        except phonenumbers.NumberParseException:
            out.append("!! %s: line %d, bad phone number: %s" % (typ, line, phone))
            return ''
        if not phonenumbers.is_valid_number(number):
            out.append("!! %s: line %d, invalid phone number: %s" % (typ, line, phone))
            return ''
        return phone

    org_infos = {}
    for typ in ['types', 'levels']:
        org_infos[typ] = OrderedDict([(t['name'], t['token']) for t in getattr(contacts, 'organization_%s' % typ)])
        if not len(org_infos[typ]):
            org_infos[typ] = OrderedDict([(u'Non défini', 'non-defini')])
    org_infos_o = copy.deepcopy(org_infos)
    # read the organization file
    lines = []
    if 'ORGS' in only:
        lines = system.read_csv(os.path.join(path, 'organizations.csv'), strip_chars=' ', strict=True)
    if lines:
        data = lines.pop(0)
        lendata = len(data)
        if lendata < 20 or data[19] != 'UID':
            return "!! Problem decoding first line: bad columns in organizations.csv ?"
    orgs = OrderedDict()
    uids = {}
    childs = {}
    idnormalizer = getUtility(IIDNormalizer)
    out = ["## ORGANIZATIONS ##\n"]
    for i, data in enumerate(lines, start=2):
        if len(data) != lendata:
            return "!! ORGS: problem line %d, invalid column number %d <> %d: %s" % (i, lendata, len(data),
                                                                                     ['%s' % cell for cell in data])
        id, idp, uid = data[0], data[1], data[19]
        if not id or id in orgs:
            return "!! ORGS: problem line %d, invalid id: %s" % (i, id)
        if uid in uids:
            return "!! ORGS: problem line %d, duplicated uid: %s, already found line %s" % (i, uid, uids[uid])
        elif uid:
            uids[uid] = 'orgs: %d' % i
        # ID;ID Par;Intitulé;Description;Type;Use par adr,Rue;Numéro;Comp adr;CP;Localité;Tél;Gsm;Fax;Courriel;Site;
        # Région;Pays;UID
        orgs[id] = {'lev': 1, 'prt': idp, 'tit': data[2], 'acronym': data[3],
                    'desc': data[4], 'upa': data[6] and int(data[6]) or '',
                    'st': data[7], 'nb': data[8], 'box': data[9], 'zip': is_zip(data[10], i, 'ORGS', data[18]),
                    'loc': data[11], 'tel': check_phone(digit(data[12]), i, 'ORGS', data[18]),
                    'mob': check_phone(digit(data[13]), i, 'ORGS', data[18]),
                    'fax': check_phone(digit(data[14]), i, 'ORGS', data[18]), 'eml': data[15],
                    'www': data[16], 'dep': data[17], 'cty': data[18], 'uid': uid}
        # stored under a parent
        if idp:
            orgs[id]['lev'] = 2
        typ = 'types'
        if idp:
            typ = 'levels'
            if idp not in childs:
                childs[idp] = []
            childs[idp].append(id)
        if data[5]:
            utyp = safe_unicode(data[5])
            if utyp not in org_infos[typ]:
                org_infos[typ][utyp] = idnormalizer.normalize(utyp)
            orgs[id]['typ'] = org_infos[typ][utyp]
        else:  # we take the first value
            orgs[id]['typ'] = org_infos[typ].values()[0]

    # updating contacts options
    for typ in ['types', 'levels']:
        if len(org_infos[typ]) != len(org_infos_o[typ]):
            out.append("Contacts parameter modification 'organization_%s'" % typ)
            if doit:
                setattr(contacts, 'organization_%s' % typ,
                        [{'name': i[0], 'token': i[1]} for i in org_infos[typ].items()])
                out.append("New value: %s" % [{'name': safe_encode(i[0]),
                                               'token': i[1]} for i in org_infos[typ].items()])
            else:
                out.append("New value will be: %s" % [{'name': safe_encode(i[0]),
                                                       'token': i[1]} for i in org_infos[typ].items()])

    # creating organization
    for i, id in enumerate(orgs, start=2):
        det = orgs[id]
        if det['lev'] == 1:
            cont = contacts
        else:
            # get the container organization, already created
            cont = orgs[det['prt']].get('obj', orgs[det['prt']]['tit'])
        action = 'create'
        if det['uid']:
            obj = uuidToObject(det['uid'])
            if not obj:
                out.append("!! %04d org: cannot find obj from uuid %s: SKIPPED" % det['uid'])
                continue
            else:
                action = 'update'
        elif ownorg and ownorg == det['tit']:
            obj = contacts['plonegroup-organization']
            action = 'update'
            det['upa'] = False
        if action == 'create':
            if doit:
                obj = api.content.create(container=cont, type='organization', title=safe_unicode(det['tit']),
                                         acronym=safe_unicode(det['acronym']),
                                         description=safe_unicode(det['desc']), organization_type=det['typ'],
                                         street=safe_unicode(det['st']), number=safe_unicode(det['nb']),
                                         additional_address_details=safe_unicode(det['box']),
                                         zip_code=safe_unicode(det['zip']), city=safe_unicode(det['loc']),
                                         phone=safe_unicode(det['tel']), cell_phone=safe_unicode(det['mob']),
                                         fax=safe_unicode(det['fax']), email=safe_unicode(det['eml']),
                                         website=safe_unicode(det['www']), region=safe_unicode(det['dep']),
                                         country=safe_unicode(det['cty']), use_parent_address=bool(det['upa']))
                det['obj'] = obj
                out.append("%04d org: new orga '%s' created in %s" % (i, safe_encode(det['tit']), safe_encode(cont)))
            else:
                out.append("%04d org: new orga '%s' will be created in %s" % (i, safe_encode(det['tit']),
                                                                              safe_encode(cont)))
        elif action == 'update':
            attrs = {'title': 'tit', 'description': 'desc', 'acronym': 'acronym', 'street': 'st', 'number': 'nb',
                     'additional_address_details': 'box', 'zip_code': 'zip', 'city': 'loc', 'phone': 'tel',
                     'cell_phone': 'mob', 'fax': 'fax', 'email': 'eml', 'website': 'www', 'region': 'dep',
                     'country': 'cty', 'organization_type': det['typ'], 'use_parent_address': bool(det['upa'])}
            change = False
            changed = []
            for attr, new_val in attrs.items():
                if attr not in ('organization_type', 'use_parent_address'):
                    new_val = safe_unicode(det[new_val])
                act_val = getattr(obj, attr)
                if act_val != new_val and not (act_val is None and new_val == u''):
                    if doit:
                        if new_val == '':
                            new_val = None
                        setattr(obj, attr, new_val)
                    change = True
                    changed.append(attr)
            if change and doit:
                obj.reindexObject()
                modified(obj)
            det['obj'] = obj
            status = ''
            if not doit:
                status = 'will be '
            if change:
                status += 'REALLY '
            else:
                status += 'not '
            out.append("%04d org: '%s' %supdated, %s" % (i, obj.absolute_url(), status, changed))

    # read the persons file
    lines = []
    if 'PERS' in only:
        lines = system.read_csv(os.path.join(path, 'persons.csv'), strip_chars=' ', strict=True)
    # ID;Nom;Prénom;Genre;Civilité;Naissance;Adr par;Rue;Numéro;Comp adr;CP;Localité;Tél;Gsm;
    # Fax;Courriel;Site;Région;Pays;Num int;UID
    if lines:
        data = lines.pop(0)
        lendata = len(data)
        if lendata < 22 or data[21] != 'UID':
            return "Problem decoding first line: bad columns in persons.csv ?"
    persons = {}
    out.append("\n## PERSONS ##\n")

    for i, data in enumerate(lines, start=2):
        if len(data) != lendata:
            return "!! PERS: problem line %d, invalid column number %d <> %d: %s" % (i, lendata, len(data),
                                                                                     ['%s' % cell for cell in data])
        id, name, fname, inum, plone_user_id, uid = data[0], data[1], data[2], data[19], data[20], data[21]
        errors = []
        try:
            upa = data[6] and int(data[6]) or ''
            phone = safe_unicode(check_phone(digit(data[12]), i, 'PERS', data[18]))
            cell_phone = safe_unicode(check_phone(digit(data[13]), i, 'PERS', data[18]))
            fax = safe_unicode(check_phone(digit(data[14]), i, 'PERS', data[18]))
            zipc = safe_unicode(is_zip(data[10], i, 'PERS', data[18]))
            gender = data[3]
            birthday = data[5] or None
        except AssertionError, ex:
            errors.append("!! PERS: problem line %d: %s" % (i, safe_encode(ex.message)))
        except Exception, ex:
            errors.append("!! PERS: problem line %d, '%s': %s" % (i, '|'.join(data), safe_encode(ex.message)))
        if not id or id in persons:
            errors.append("!! PERS: problem line %d, invalid id: %s" % (i, id))
        if uid in uids:
            errors.append("!! PERS: problem line %d, duplicated uid: %s, already found line %s" % (i, uid, uids[uid]))
        elif uid:
            uids[uid] = 'pers: %d' % i
        if errors:
            if doit:
                return '\n'.join(errors)
            out.append('\n'.join(errors))

        persons[id] = {}
        action = 'create'
        if uid:
            obj = uuidToObject(uid)
            if not obj:
                out.append("!! PERS %04d: cannot find obj from uuid %s: SKIPPED" % uid)
                continue
            else:
                action = 'update'
        elif inum:
            brains = api.content.find(portal_type='person', internal_number=inum)
            if len(brains) == 1:
                obj = brains[0].getObject()
                action = 'update'
            elif len(brains) > 1:
                out.append("!! PERS %04d: multiple persons found with int number '%s': SKIPPED (%s)" % (i, inum,
                           ','.join([b.getPath() for b in brains])))
                continue
        if action == 'create':
            if doit:
                real_id = new_id = idnormalizer.normalize(safe_encode('%s-%s' % (fname, name)))
                count = 0
                while real_id in contacts:
                    count += 1
                    real_id = '%s-%d' % (new_id, count)

                obj = api.content.create(container=contacts, type='person', id=real_id, lastname=safe_unicode(name),
                                         firstname=safe_unicode(fname), gender=gender,
                                         person_title=safe_unicode(data[4]), birthday=birthday,
                                         street=safe_unicode(data[7]), number=safe_unicode(data[8]),
                                         additional_address_details=safe_unicode(data[9]),
                                         zip_code=zipc, city=safe_unicode(data[11]),
                                         phone=phone, cell_phone=cell_phone, fax=fax, email=safe_unicode(data[15]),
                                         website=safe_unicode(data[16]), region=safe_unicode(data[17]),
                                         country=safe_unicode(data[18]), use_parent_address=bool(upa),
                                         userid=plone_user_id)
                if inum and IInternalNumberBehavior.providedBy(obj):
                    obj.internal_number = inum
                    obj.reindexObject(idxs=['internal_number', 'SearchableText'])
                out.append("%04d pers: new person '%s %s' created" % (i, safe_encode(name), safe_encode(fname)))
                persons[id]['obj'] = obj
            else:
                out.append("%04d pers: new person '%s %s' will be created" % (i, safe_encode(name), safe_encode(fname)))
        elif action == 'update':
            attrs = {'lastname': 1, 'firstname': 2, 'gender': gender, 'person_title': 4, 'birthday': birthday,
                     'street': 7, 'number': 8, 'additional_address_details': 9, 'zip_code': zipc, 'city': 11,
                     'phone': phone, 'cell_phone': cell_phone, 'fax': fax, 'email': 15, 'website': 16, 'region': 17,
                     'country': 18, 'use_parent_address': bool(upa)}
            change = False
            changed = []
            for attr, new_val in attrs.items():
                if attr not in ('gender', 'birthday', 'use_parent_address', 'phone', 'cell_phone', 'fax', 'zip_code'):
                    new_val = safe_unicode(data[new_val])
                act_val = getattr(obj, attr)
                if act_val != new_val and not (act_val is None and new_val == u''):
                    if doit:
                        if new_val == '':
                            new_val = None
                        setattr(obj, attr, new_val)
                    change = True
                    changed.append(attr)
            if change and doit:
                obj.reindexObject()
                modified(obj)
            persons[id]['obj'] = obj
            status = ''
            if not doit:
                status = 'will be '
            if change:
                status += 'REALLY '
            else:
                status += 'not '
            out.append("%04d pers: '%s' %supdated, %s" % (i, obj.absolute_url(), status, changed))

    # read the heldpositions file
    lines = []
    if 'HP' in only:
        lines = system.read_csv(os.path.join(path, 'heldpositions.csv'), strip_chars=' ', strict=True)
    # ID;ID person;ID org;ID fct;Intitulé fct;position_type;Début fct;Fin fct;Adr par;Rue;Numéro;Comp adr;
    # CP;Localité;Tél;Gsm;Fax;Courriel;Site;Région;Pays;UID
    if lines:
        data = lines.pop(0)
        lendata = len(data)
        if lendata < 24 or data[23] != 'UID':
            return "Problem decoding first line: bad columns in heldpositions.csv ?"
        intids = getUtility(IIntIds)
    hps = {}
    out.append("\n## HELD POSITIONS ##\n")
    for i, data in enumerate(lines, start=2):
        if len(data) != lendata:
            return "!! HP: problem line %d, invalid column number %d <> %d: %s" % (i, lendata, len(data),
                                                                                   ['%s' % cell for cell in data])
        id, pid, oid, position_type, title, uid = data[0], data[1], data[2], data[3], data[4], data[23]
        errors = []
        try:
            start = data[5] or None
            end = data[6] or None
            phone = safe_unicode(check_phone(digit(data[13]), i, 'PERS', data[19]))
            cell_phone = safe_unicode(check_phone(digit(data[14]), i, 'PERS', data[19]))
            fax = safe_unicode(check_phone(digit(data[15]), i, 'PERS', data[19]))
            upa = data[7] and int(data[7]) or ''
            zipc = safe_unicode(is_zip(data[11], i, 'HP', data[19]))
        except AssertionError, ex:
            errors.append("!! HP: problem line %d: %s" % (i, safe_encode(ex.message)))
        except Exception, ex:
            errors.append("!! HP: problem line %d, '%s': %s" % (i, '|'.join(data), safe_encode(ex.message)))
        if not id or id in hps:
            errors.append("!! HP: problem line %d, invalid id: %s" % (i, id))
        if not pid:
            errors.append("!! HP: problem line %d, invalid person id: %s" % (i, pid))
        if not oid:
            errors.append("!! HP: problem line %d, invalid org id: %s" % (i, oid))
        if uid in uids:
            errors.append("!! HP: problem line %d, duplicated uid: %s, already found line %s" % (i, uid, uids[uid]))
        elif uid:
            uids[uid] = 'hp: %d' % i
        if errors:
            if doit:
                return '\n'.join(errors)
            out.append('\n'.join(errors))

        hps[id] = {}
        action = 'create'
        if uid:
            obj = uuidToObject(uid)
            if not obj:
                out.append("!! HP %04d: cannot find obj from uuid %s: SKIPPED" % uid)
                continue
            else:
                action = 'update'
        if pid in persons:
            if 'obj' in persons[pid]:
                pers = persons[pid]['obj']
            elif not doit:
                pers = portal  # without doit in creation mode, obj not there => take portal
            else:
                pers = None
        else:
            out.append("!! HP %04d: person not found for id '%s': SKIPPED" % (i, pid))
            continue
        if oid in orgs:
            if 'obj' in orgs[oid]:
                org = orgs[oid]['obj']
            elif not doit:
                org = portal  # without doit in creation mode, obj not there => take portal
            else:
                org = None
        else:
            out.append("!! HP %04d: org not found for id '%s': SKIPPED" % (i, oid))
            continue
        if action == 'create':
            if doit:
                real_id = new_id = idnormalizer.normalize(safe_encode('%s-%s' % (
                    safe_unicode(title) or safe_unicode(position_type), org.title)))
                count = 0
                while real_id in pers:
                    count += 1
                    real_id = '%s-%d' % (new_id, count)

                obj = api.content.create(container=pers, type='held_position', id=real_id,
                                         position=RelationValue(intids.getId(org)),
                                         position_type=position_type,
                                         label=safe_unicode(title), start_date=start, end_date=end,
                                         street=safe_unicode(data[8]), number=safe_unicode(data[9]),
                                         additional_address_details=safe_unicode(data[10]),
                                         zip_code=zipc, city=safe_unicode(data[12]),
                                         phone=phone, cell_phone=cell_phone, fax=fax, email=safe_unicode(data[16]),
                                         website=safe_unicode(data[17]), region=safe_unicode(data[18]),
                                         country=safe_unicode(data[19]), use_parent_address=bool(upa),
                                         defaults=data[20].split('|'), usages=data[21].split('|'),
                                         signature_number=data[22] or None)
                out.append("%04d hp: new hp '%s' for '%s' created" % (i, safe_encode(title), pers.Title()))
                hps[id]['obj'] = obj
            else:
                out.append("%04d hp: new hp '%s %s' will be created" % (i, safe_encode(title), pers.Title()))
        elif action == 'update':
            intid = intids.getId(org)
            new_pos = obj.position
            if new_pos and new_pos.to_id != intid:
                new_pos = RelationValue(intid)
            attrs = {'position': new_pos, 'label': 4, 'start_date': start, 'end_date': end,
                     'street': 8, 'number': 9, 'additional_address_details': 10, 'zip_code': zipc, 'city': 12,
                     'phone': phone, 'cell_phone': cell_phone, 'fax': fax, 'email': 16, 'website': 17, 'region': 18,
                     'country': 19, 'use_parent_address': bool(upa)}
            change = False
            changed = []
            for attr, new_val in attrs.items():
                if attr not in ('position', 'start_date', 'end_date', 'phone', 'cell_phone', 'fax',
                                'use_parent_address', 'zip_code'):
                    new_val = safe_unicode(data[new_val])
                act_val = getattr(obj, attr)
                if act_val != new_val and not (act_val is None and new_val == u''):
                    if doit:
                        if new_val == '':
                            new_val = None
                        setattr(obj, attr, new_val)
                    change = True
                    changed.append(attr)
            if change and doit:
                obj.reindexObject()
                modified(obj)
            hps[id]['obj'] = obj
            status = ''
            if not doit:
                status = 'will be '
            if change:
                status += 'REALLY '
            else:
                status += 'not '
            out.append("%04d hp: '%s' %supdated, %s" % (i, obj.absolute_url(), status, changed))

    return '\n'.join(out)



# -*- coding: utf-8 -*-

from imio.helpers.content import safe_delattr
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from Products.ZCatalog.ProgressHandler import ZLogHandler


BARCODE_INSERTED_ATTR_ID = '_barcode_inserted'


class Migrate_To_4210(Migrator):

    def _removeBarcodeInsertedAttrOnAnnexes(self):
        """ """
        logger.info('Removing attribute "_barcode_inserted" on every annexes...')
        brains = self.catalog(portal_type=['annex', 'annexDecision'])
        i = 0
        pghandler = ZLogHandler(steps=1000)
        pghandler.init('Removing attribute "_barcode_inserted" on every annexes...', len(brains))
        for brain in brains:
            i += 1
            pghandler.report(i)
            if brain.scan_id:
                safe_delattr(brain.getObject(), BARCODE_INSERTED_ATTR_ID)
        pghandler.finish()
        logger.info('Done.')

    def run(self, extra_omitted=[], from_migration_to_4200=False):

        logger.info('Migrating to PloneMeeting 4210...')

        self._removeBarcodeInsertedAttrOnAnnexes()
        self.updateHolidays()  # holidays 2024 were added

        logger.info('Migrating to PloneMeeting 4210... Done.')


def migrate(context):
    '''This migration function will:

       1) Remove attribute "_barcode_inserted" from annexes.
    '''
    migrator = Migrate_To_4210(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-

from plone.app.controlpanel.mail import MailControlPanelAdapter
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from Products.PloneMeeting.utils import get_public_url


class Migrate_To_4109(Migrator):

    def force_email_from_address(self):
        logger.info("Fixing email from address...")
        mail_panel_adapter = MailControlPanelAdapter(self.portal)
        smtp_host = mail_panel_adapter.smtp_host or u''
        if smtp_host.strip() != u'localhost':
            logger.info("Bypassing, smtp_host is not localhost...")
        else:
            public_url = get_public_url(self.portal)
            mail_address = public_url.replace("https://", "")
            if "-pm" not in mail_address:
                logger.info("Bypassing, \"-pm\" not found in public_url...")
            else:
                index = mail_address.index("-pm")
                mail_address = "%s-delib@imio.be" % mail_address[:index]
                logger.info("Setting \"{0}\" as email_from_address.".format(mail_address))
                mail_panel_adapter.set_email_from_address(mail_address)
        logger.info('Done.')

    def run(self, from_migration_to_41=False):
        logger.info('Migrating to PloneMeeting 4109...')
        self.force_email_from_address()


def migrate(context):
    '''This migration function will:

       1) Force mail sender address.
    '''
    migrator = Migrate_To_4109(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from AccessControl import Unauthorized
from plone import api
from Products.PloneMeeting.indexes import DELAYAWARE_ROW_ID_PATTERN
from Products.PloneMeeting.indexes import REAL_ORG_UID_PATTERN

import logging


logger = logging.getLogger('PloneMeeting')


def commonChecks(group_id):
    """ """
    portal = api.portal.get()
    tool = api.portal.get_tool('portal_plonemeeting')
    # some verifications, must be Manager and new_proposing_group_id must be valid
    if not tool.isManager(portal, realManagers=True):
        raise Unauthorized
    if group_id not in tool.objectIds('MeetingGroup'):
        raise KeyError('{0} is not a valid proposingGroup id!'.format(group_id))


def changeProposingGroupFor(old_proposing_group_id, new_proposing_group_id, ):
    '''Change items having p_old_proposing_group_id by p_new_proposing_group_id.'''
    commonChecks(new_proposing_group_id)
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog(meta_type='MeetingItem',
                     getProposingGroup=old_proposing_group_id)
    # if reindexObject is called without idxs, it is notifyModified...
    catalog_indexes = catalog.Indexes.keys()
    for brain in brains:
        obj = brain.getObject()
        obj.setProposingGroup(new_proposing_group_id)
        obj.update_local_roles()
        obj.reindexObject(idxs=catalog_indexes)
        logger.info('Obj at {0} proposingGroup is now {1}'.format('/'.join(obj.getPhysicalPath()),
                                                                  new_proposing_group_id))


def lookForCopyGroups(group_id):
    """Check if some copy groups of a given p_group_id is used by any items in the application."""
    commonChecks(group_id)
    tool = api.portal.get_tool('portal_plonemeeting')
    catalog = api.portal.get_tool('portal_catalog')
    group = getattr(tool, group_id)
    suffixes = group.get_all_suffixes(group_id)
    res = {}
    for suffix in suffixes:
        ploneGroupId = group.getPloneGroupId(suffix)
        brains = catalog(getCopyGroups=ploneGroupId)
        if brains:
            res[suffix] = [brain.getURL() for brain in brains]

    output = []
    for k, v in res.items():
        output.append(k)
        for value in v:
            output.append(value)
    return '\n'.join(output)


def lookForAdvisers(group_id):
    """Check if given p_group_id is used as adviser."""
    commonChecks(group_id)
    tool = api.portal.get_tool('portal_plonemeeting')
    catalog = api.portal.get_tool('portal_catalog')
    # get row_ids of customAdvisers the group_id is used with
    delayAwareCustomAdvisersRowIds = []
    for mc in tool.objectValues('MeetingConfig'):
        delayAwareCustomAdvisersRowIds += [customAdviser['row_id'] for customAdviser in mc.getCustomAdvisers()
                                           if (customAdviser['group'] == group_id and customAdviser['delay'])]
    delay_aware_values = [DELAYAWARE_ROW_ID_PATTERN.format(delay_aware_group_id)
                          for delay_aware_group_id in delayAwareCustomAdvisersRowIds]
    real_group_id_value = REAL_ORG_UID_PATTERN.format(group_id)

    res = []
    brains = catalog(indexAdvisers=delay_aware_values + [real_group_id_value])
    if brains:
        res = [brain.getURL() for brain in brains]

    return '\n'.join(res)
